"""Tests for update checker functionality."""

import json
import tempfile
from datetime import datetime, timedelta
from pathlib import Path
from unittest.mock import Mock, patch
from urllib.error import URLError

from src.rxiv_maker.utils.update_checker import UpdateChecker, get_update_checker


class TestUpdateChecker:
    """Test UpdateChecker class."""

    def setup_method(self):
        """Set up test environment."""
        self.package_name = "test-package"
        self.current_version = "1.0.0"

    def test_init(self):
        """Test UpdateChecker initialization."""
        checker = UpdateChecker(self.package_name, self.current_version)
        assert checker.package_name == self.package_name
        assert checker.current_version == self.current_version
        assert checker.pypi_url == f"https://pypi.org/pypi/{self.package_name}/json"

    def test_should_check_for_updates_env_var_opt_out(self):
        """Test opt-out via environment variable."""
        checker = UpdateChecker(self.package_name, self.current_version)

        with patch.dict("os.environ", {"RXIV_NO_UPDATE_CHECK": "1"}):
            assert not checker.should_check_for_updates()

        with patch.dict("os.environ", {"NO_UPDATE_NOTIFIER": "1"}):
            assert not checker.should_check_for_updates()

    def test_should_check_for_updates_time_interval(self):
        """Test time interval checking."""
        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            # No cache file - should check
            assert checker.should_check_for_updates()

            # Recent check - should not check
            recent_cache = {
                "last_check": datetime.now().isoformat(),
                "latest_version": "1.1.0",
                "update_available": True,
            }
            with open(cache_file, "w") as f:
                json.dump(recent_cache, f)

            assert not checker.should_check_for_updates()

            # Old check - should check
            old_cache = {
                "last_check": (datetime.now() - timedelta(days=2)).isoformat(),
                "latest_version": "1.1.0",
                "update_available": True,
            }
            with open(cache_file, "w") as f:
                json.dump(old_cache, f)

            assert checker.should_check_for_updates()

    def test_load_cache_valid(self):
        """Test loading valid cache."""
        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            test_data = {"test": "data"}
            with open(cache_file, "w") as f:
                json.dump(test_data, f)

            result = checker._load_cache()
            assert result == test_data

    def test_load_cache_invalid(self):
        """Test loading invalid cache."""
        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            # Invalid JSON
            with open(cache_file, "w") as f:
                f.write("invalid json")

            result = checker._load_cache()
            assert result is None

            # Non-existent file
            cache_file.unlink()
            result = checker._load_cache()
            assert result is None

    def test_save_cache(self):
        """Test saving cache."""
        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            test_data = {"test": "data"}
            checker._save_cache(test_data)

            with open(cache_file) as f:
                result = json.load(f)

            assert result == test_data

    @patch("src.rxiv_maker.utils.update_checker.urlopen")
    def test_fetch_latest_version_success(self, mock_urlopen):
        """Test successful version fetching."""
        mock_response = Mock()
        mock_response.read.return_value = b'{"info": {"version": "1.2.0"}}'
        mock_urlopen.return_value.__enter__.return_value = mock_response

        checker = UpdateChecker(self.package_name, self.current_version)
        result = checker._fetch_latest_version()

        assert result == "1.2.0"
        mock_urlopen.assert_called_once_with(checker.pypi_url, timeout=5)

    @patch("src.rxiv_maker.utils.update_checker.urlopen")
    def test_fetch_latest_version_failure(self, mock_urlopen):
        """Test failed version fetching."""
        mock_urlopen.side_effect = URLError("Network error")

        checker = UpdateChecker(self.package_name, self.current_version)
        result = checker._fetch_latest_version()

        assert result is None

    def test_compare_versions_with_packaging(self):
        """Test version comparison with packaging library."""
        checker = UpdateChecker(self.package_name, self.current_version)

        # Newer version
        assert checker._compare_versions("1.0.0", "1.1.0")
        assert checker._compare_versions("1.0.0", "2.0.0")

        # Same version
        assert not checker._compare_versions("1.0.0", "1.0.0")

        # Older version
        assert not checker._compare_versions("1.1.0", "1.0.0")

    def test_compare_versions_fallback(self):
        """Test version comparison fallback."""
        checker = UpdateChecker(self.package_name, self.current_version)

        with patch("src.rxiv_maker.utils.update_checker.pkg_version", None):
            # Should fall back to string comparison
            assert checker._compare_versions("1.0.0", "1.1.0")  # Different = newer
            assert not checker._compare_versions("1.0.0", "1.0.0")  # Same = not newer

    @patch("src.rxiv_maker.utils.update_checker.detect_install_method")
    def test_get_update_notification_available(self, mock_detect):
        """Test update notification when update is available."""
        mock_detect.return_value = "pip"

        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            cache_data = {
                "last_check": datetime.now().isoformat(),
                "latest_version": "1.2.0",
                "current_version": "1.0.0",
                "update_available": True,
            }
            with open(cache_file, "w") as f:
                json.dump(cache_data, f)

            notification = checker.get_update_notification()
            assert notification is not None
            # Check for version numbers (may have Rich markup)
            assert "v1.0.0" in notification
            assert "v1.2.0" in notification
            assert "→" in notification or "->" in notification
            assert "pip install --upgrade" in notification
            assert "full details" in notification.lower() or "Full details" in notification

    def test_get_update_notification_not_available(self):
        """Test update notification when no update is available."""
        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            cache_data = {
                "last_check": datetime.now().isoformat(),
                "latest_version": "1.0.0",
                "current_version": "1.0.0",
                "update_available": False,
            }
            with open(cache_file, "w") as f:
                json.dump(cache_data, f)

            notification = checker.get_update_notification()
            assert notification is None

    def test_get_update_notification_same_version_stale_cache(self):
        """Test no notification when cache says update available but versions match (after upgrade)."""
        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            # User has upgraded to latest but cache still says update available
            checker = UpdateChecker(self.package_name, "1.2.0")  # Current = latest
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            cache_data = {
                "last_check": datetime.now().isoformat(),
                "latest_version": "1.2.0",  # Same as current
                "current_version": "1.0.0",  # Stale cached version
                "update_available": True,  # Stale flag
            }
            with open(cache_file, "w") as f:
                json.dump(cache_data, f)

            notification = checker.get_update_notification()
            assert notification is None  # Should not show notification when versions match

    @patch("src.rxiv_maker.utils.update_checker.urlopen")
    def test_force_check(self, mock_urlopen):
        """Test force check functionality."""
        mock_response = Mock()
        mock_response.read.return_value = b'{"info": {"version": "1.2.0"}}'
        mock_urlopen.return_value.__enter__.return_value = mock_response

        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_dir / "update_cache.json"

            update_available, latest_version = checker.force_check()

            assert update_available
            assert latest_version == "1.2.0"

            # Check that cache was updated
            cache_data = checker._load_cache()
            assert cache_data["update_available"]
            assert cache_data["latest_version"] == "1.2.0"

    def test_check_for_updates_async(self):
        """Test async update checking."""
        checker = UpdateChecker(self.package_name, self.current_version)

        with patch.object(checker, "should_check_for_updates", return_value=False):
            # Should not start thread if checking is disabled
            with patch("threading.Thread") as mock_thread:
                checker.check_for_updates_async()
                mock_thread.assert_not_called()

        with patch.object(checker, "should_check_for_updates", return_value=True):
            # Should start thread if checking is enabled
            with patch("threading.Thread") as mock_thread:
                checker.check_for_updates_async()
                mock_thread.assert_called_once()
                mock_thread.return_value.start.assert_called_once()


class TestGlobalFunctions:
    """Test global convenience functions."""

    def test_get_update_checker_singleton(self):
        """Test that get_update_checker returns singleton."""
        checker1 = get_update_checker()
        checker2 = get_update_checker()
        assert checker1 is checker2

    @patch("src.rxiv_maker.utils.update_checker.get_update_checker")
    def test_check_for_updates_async(self, mock_get_checker):
        """Test global check_for_updates_async function."""
        mock_checker = Mock()
        mock_get_checker.return_value = mock_checker

        from src.rxiv_maker.utils.update_checker import check_for_updates_async

        check_for_updates_async()

        mock_checker.check_for_updates_async.assert_called_once()

    @patch("src.rxiv_maker.utils.update_checker.get_update_checker")
    def test_show_update_notification(self, mock_get_checker):
        """Test global show_update_notification function."""
        mock_checker = Mock()
        mock_get_checker.return_value = mock_checker

        from src.rxiv_maker.utils.update_checker import show_update_notification

        show_update_notification()

        mock_checker.show_update_notification.assert_called_once()

    @patch("src.rxiv_maker.utils.update_checker.get_update_checker")
    def test_force_update_check(self, mock_get_checker):
        """Test global force_update_check function."""
        mock_checker = Mock()
        mock_checker.force_check.return_value = (True, "1.2.0")
        mock_get_checker.return_value = mock_checker

        from src.rxiv_maker.utils.update_checker import force_update_check

        result = force_update_check()

        assert result == (True, "1.2.0")
        mock_checker.force_check.assert_called_once()


class TestInstallDetectionIntegration:
    """Test installation detection integration with update notifications."""

    def setup_method(self):
        """Set up test environment."""
        self.package_name = "test-package"
        self.current_version = "1.0.0"

    @patch("src.rxiv_maker.utils.update_checker.detect_install_method")
    def test_update_notification_pip(self, mock_detect):
        """Test update notification for pip installation."""
        mock_detect.return_value = "pip"

        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            cache_data = {
                "last_check": datetime.now().isoformat(),
                "latest_version": "1.2.0",
                "current_version": "1.0.0",
                "update_available": True,
            }
            with open(cache_file, "w") as f:
                json.dump(cache_data, f)

            notification = checker.get_update_notification()
            assert notification is not None
            assert "pip install --upgrade" in notification
            assert "Installed via" in notification and "pip" in notification

    @patch("src.rxiv_maker.utils.update_checker.detect_install_method")
    def test_update_notification_homebrew(self, mock_detect):
        """Test update notification for Homebrew installation."""
        mock_detect.return_value = "homebrew"

        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            cache_data = {
                "last_check": datetime.now().isoformat(),
                "latest_version": "1.2.0",
                "current_version": "1.0.0",
                "update_available": True,
            }
            with open(cache_file, "w") as f:
                json.dump(cache_data, f)

            notification = checker.get_update_notification()
            assert notification is not None
            assert "brew update && brew upgrade" in notification
            assert "Installed via" in notification and "Homebrew" in notification

    @patch("src.rxiv_maker.utils.update_checker.detect_install_method")
    def test_update_notification_uv(self, mock_detect):
        """Test update notification for uv installation."""
        mock_detect.return_value = "uv"

        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            cache_data = {
                "last_check": datetime.now().isoformat(),
                "latest_version": "1.2.0",
                "current_version": "1.0.0",
                "update_available": True,
            }
            with open(cache_file, "w") as f:
                json.dump(cache_data, f)

            notification = checker.get_update_notification()
            assert notification is not None
            assert "uv tool upgrade" in notification
            assert "Installed via" in notification and "uv tool" in notification

    @patch("src.rxiv_maker.utils.update_checker.detect_install_method")
    def test_update_notification_pipx(self, mock_detect):
        """Test update notification for pipx installation."""
        mock_detect.return_value = "pipx"

        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()
            cache_file = cache_dir / "update_cache.json"

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_file

            cache_data = {
                "last_check": datetime.now().isoformat(),
                "latest_version": "1.2.0",
                "current_version": "1.0.0",
                "update_available": True,
            }
            with open(cache_file, "w") as f:
                json.dump(cache_data, f)

            notification = checker.get_update_notification()
            assert notification is not None
            assert "pipx upgrade" in notification
            assert "Installed via" in notification and "pipx" in notification

    @patch("src.rxiv_maker.utils.update_checker.detect_install_method")
    @patch("src.rxiv_maker.utils.update_checker.check_homebrew_update")
    def test_homebrew_preferred_check(self, mock_homebrew_check, mock_detect):
        """Test that Homebrew update checker is used for Homebrew installations."""
        mock_detect.return_value = "homebrew"
        mock_homebrew_check.return_value = (True, "1.2.0")

        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_dir / "update_cache.json"

            checker._check_and_cache_update()

            # Verify Homebrew checker was called
            mock_homebrew_check.assert_called_once_with(self.current_version)

            # Verify cache was updated with Homebrew results
            cache_data = checker._load_cache()
            assert cache_data["update_available"]
            assert cache_data["latest_version"] == "1.2.0"

    @patch("src.rxiv_maker.utils.update_checker.detect_install_method")
    @patch("src.rxiv_maker.utils.update_checker.check_homebrew_update")
    @patch("src.rxiv_maker.utils.update_checker.UpdateChecker._fetch_latest_version")
    def test_fallback_to_pypi_for_non_homebrew(self, mock_fetch, mock_homebrew_check, mock_detect):
        """Test that PyPI is used for non-Homebrew installations."""
        mock_detect.return_value = "pip"
        mock_fetch.return_value = "1.2.0"

        with tempfile.TemporaryDirectory() as tmpdir:
            cache_dir = Path(tmpdir) / ".rxiv"
            cache_dir.mkdir()

            checker = UpdateChecker(self.package_name, self.current_version)
            checker.cache_dir = cache_dir
            checker.cache_file = cache_dir / "update_cache.json"

            checker._check_and_cache_update()

            # Verify Homebrew checker was NOT called
            mock_homebrew_check.assert_not_called()

            # Verify PyPI was called
            mock_fetch.assert_called_once()

            # Verify cache was updated
            cache_data = checker._load_cache()
            assert cache_data["latest_version"] == "1.2.0"


class TestUpdateCheckerChangelogIntegration:
    """Tests for changelog integration in update notifications."""

    package_name = "rxiv-maker"
    current_version = "1.12.0"

    @patch("rxiv_maker.utils.update_checker.fetch_and_format_changelog")
    @patch("rxiv_maker.utils.update_checker.detect_install_method")
    def test_update_notification_includes_changelog(self, mock_detect, mock_changelog, tmp_path):
        """Test that update notification includes changelog summary."""
        from rxiv_maker.utils.update_checker import UpdateChecker

        mock_detect.return_value = "pip"
        mock_changelog.return_value = (
            "⚠️  BREAKING CHANGES:\n  • Config format changed\n\nWhat's New:\n  ✨ New feature",
            None,
        )

        cache_dir = tmp_path / "cache"
        cache_dir.mkdir()

        checker = UpdateChecker(self.package_name, self.current_version)
        checker.cache_dir = cache_dir
        checker.cache_file = cache_dir / "update_cache.json"

        # Set up cache with update available
        cache_data = {
            "last_check": datetime.now().isoformat(),
            "latest_version": "1.13.0",
            "current_version": self.current_version,
            "update_available": True,
        }
        checker._save_cache(cache_data)

        # Get notification
        notification = checker.get_update_notification()

        assert notification is not None
        assert "v1.12.0" in notification
        assert "v1.13.0" in notification
        assert "→" in notification or "->" in notification
        assert "BREAKING CHANGES" in notification
        assert "New feature" in notification
        mock_changelog.assert_called_once()

    @patch("rxiv_maker.utils.update_checker.fetch_and_format_changelog")
    @patch("rxiv_maker.utils.update_checker.detect_install_method")
    def test_update_notification_without_changelog_on_error(self, mock_detect, mock_changelog, tmp_path):
        """Test that notification works even if changelog fetch fails."""
        from rxiv_maker.utils.update_checker import UpdateChecker

        mock_detect.return_value = "pip"
        mock_changelog.return_value = (None, "Network error")

        cache_dir = tmp_path / "cache"
        cache_dir.mkdir()

        checker = UpdateChecker(self.package_name, self.current_version)
        checker.cache_dir = cache_dir
        checker.cache_file = cache_dir / "update_cache.json"

        # Set up cache with update available
        cache_data = {
            "last_check": datetime.now().isoformat(),
            "latest_version": "1.13.0",
            "current_version": self.current_version,
            "update_available": True,
        }
        checker._save_cache(cache_data)

        # Get notification
        notification = checker.get_update_notification()

        assert notification is not None
        assert "v1.12.0" in notification
        assert "v1.13.0" in notification
        # Should still show basic notification without changelog
        assert "pip install --upgrade" in notification

    @patch("rxiv_maker.utils.update_checker.fetch_and_format_changelog")
    @patch("rxiv_maker.utils.update_checker.detect_install_method")
    def test_changelog_caching(self, mock_detect, mock_changelog, tmp_path):
        """Test that changelog is cached to avoid repeated fetches."""
        from rxiv_maker.utils.update_checker import UpdateChecker

        mock_detect.return_value = "pip"
        mock_changelog.return_value = ("What's New:\n  ✨ Feature", None)

        cache_dir = tmp_path / "cache"
        cache_dir.mkdir()

        checker = UpdateChecker(self.package_name, self.current_version)
        checker.cache_dir = cache_dir
        checker.cache_file = cache_dir / "update_cache.json"

        # Set up cache with update available
        cache_data = {
            "last_check": datetime.now().isoformat(),
            "latest_version": "1.13.0",
            "current_version": self.current_version,
            "update_available": True,
        }
        checker._save_cache(cache_data)

        # First call should fetch changelog
        notification1 = checker.get_update_notification()
        assert mock_changelog.call_count == 1

        # Second call should use cached changelog
        notification2 = checker.get_update_notification()
        # Should still be 1, not 2, because it's cached
        assert mock_changelog.call_count == 1

        # Both notifications should be identical
        assert notification1 == notification2

    @patch("rxiv_maker.utils.update_checker.fetch_and_format_changelog")
    @patch("rxiv_maker.utils.update_checker.detect_install_method")
    def test_changelog_multi_version_update(self, mock_detect, mock_changelog, tmp_path):
        """Test changelog for multi-version updates."""
        from rxiv_maker.utils.update_checker import UpdateChecker

        mock_detect.return_value = "pip"
        # Simulate multi-version changelog
        multi_version_summary = """⚠️  BREAKING CHANGES:
  • Breaking change in v1.12.0

What's New:

  v1.13.0 (2025-11-24):
    ✨ Citation styles
    ✨ DOI resolution

  v1.12.1 (2025-11-20):
    🐛 Bug fixes

  v1.12.0 (2025-11-19):
    ✨ New features
"""
        mock_changelog.return_value = (multi_version_summary, None)

        cache_dir = tmp_path / "cache"
        cache_dir.mkdir()

        checker = UpdateChecker(self.package_name, "1.11.0")  # Older version
        checker.cache_dir = cache_dir
        checker.cache_file = cache_dir / "update_cache.json"

        # Set up cache with update available
        cache_data = {
            "last_check": datetime.now().isoformat(),
            "latest_version": "1.13.0",
            "current_version": "1.11.0",
            "update_available": True,
        }
        checker._save_cache(cache_data)

        # Get notification
        notification = checker.get_update_notification()

        assert notification is not None
        assert "v1.11.0" in notification
        assert "v1.13.0" in notification
        assert "→" in notification or "->" in notification
        assert "v1.12.1" in notification
        assert "v1.12.0" in notification
        assert "BREAKING CHANGES" in notification

    @patch("rxiv_maker.utils.update_checker.fetch_and_format_changelog")
    @patch("rxiv_maker.utils.update_checker.detect_install_method")
    def test_changelog_with_breaking_changes_prominence(self, mock_detect, mock_changelog, tmp_path):
        """Test that breaking changes are prominently displayed."""
        from rxiv_maker.utils.update_checker import UpdateChecker

        mock_detect.return_value = "pip"
        mock_changelog.return_value = (
            "⚠️  BREAKING CHANGES:\n  • API changed\n  • Config format changed\n\nWhat's New:\n  ✨ Features",
            None,
        )

        cache_dir = tmp_path / "cache"
        cache_dir.mkdir()

        checker = UpdateChecker(self.package_name, self.current_version)
        checker.cache_dir = cache_dir
        checker.cache_file = cache_dir / "update_cache.json"

        # Set up cache with update available
        cache_data = {
            "last_check": datetime.now().isoformat(),
            "latest_version": "1.13.0",
            "current_version": self.current_version,
            "update_available": True,
        }
        checker._save_cache(cache_data)

        # Get notification
        notification = checker.get_update_notification()

        assert notification is not None
        # Breaking changes should appear before "What's New"
        breaking_pos = notification.index("BREAKING")
        whats_new_pos = notification.index("What's New")
        assert breaking_pos < whats_new_pos



"""Tests for title synchronization validation in SyntaxValidator."""

import tempfile
from pathlib import Path

import pytest
import yaml

from rxiv_maker.validators.syntax_validator import SyntaxValidator


@pytest.fixture
def temp_manuscript_with_mismatch():
    """Create a temporary manuscript with mismatched titles."""
    with tempfile.TemporaryDirectory() as tmpdir:
        manuscript_path = Path(tmpdir)

        # Create config with one title
        config_path = manuscript_path / "00_CONFIG.yml"
        with open(config_path, "w") as f:
            yaml.dump({"title": "Config Title"}, f)

        # Create main with different title
        main_path = manuscript_path / "01_MAIN.md"
        with open(main_path, "w") as f:
            f.write("# Main Title\n\n## Introduction\nContent.\n")

        yield manuscript_path


@pytest.fixture
def temp_manuscript_with_match():
    """Create a temporary manuscript with matching titles."""
    with tempfile.TemporaryDirectory() as tmpdir:
        manuscript_path = Path(tmpdir)

        # Create config
        config_path = manuscript_path / "00_CONFIG.yml"
        with open(config_path, "w") as f:
            yaml.dump({"title": "Same Title"}, f)

        # Create main with same title
        main_path = manuscript_path / "01_MAIN.md"
        with open(main_path, "w") as f:
            f.write("# Same Title\n\n## Introduction\nContent.\n")

        yield manuscript_path


@pytest.fixture
def temp_manuscript_with_auto_title():
    """Create a temporary manuscript with auto-generated title."""
    with tempfile.TemporaryDirectory() as tmpdir:
        manuscript_path = Path(tmpdir)

        # Create config
        config_path = manuscript_path / "00_CONFIG.yml"
        with open(config_path, "w") as f:
            yaml.dump({"title": "Config Title"}, f)

        # Create main with auto-generated title (different from config)
        main_path = manuscript_path / "01_MAIN.md"
        with open(main_path, "w") as f:
            f.write(
                "<!-- Title auto-synced from config (00_CONFIG.yml) by rxiv-maker -->\n"
                "# Old Auto Title\n\n"
                "## Introduction\nContent.\n"
            )

        yield manuscript_path


class TestTitleSyncValidation:
    """Tests for title sync validation in SyntaxValidator."""

    def test_validator_detects_title_mismatch(self, temp_manuscript_with_mismatch):
        """Test that validator detects title mismatch."""
        validator = SyntaxValidator(str(temp_manuscript_with_mismatch))
        result = validator.validate()

        # Should have at least one error
        assert len(result.errors) > 0

        # Check that there's a title_mismatch error
        title_errors = [e for e in result.errors if e.error_code == "title_mismatch"]
        assert len(title_errors) == 1

        error = title_errors[0]
        assert "Config Title" in error.message or (error.context and "Config Title" in error.context)
        assert "Main Title" in error.message or (error.context and "Main Title" in error.context)

    def test_validator_passes_with_matching_titles(self, temp_manuscript_with_match):
        """Test that validator passes when titles match."""
        validator = SyntaxValidator(str(temp_manuscript_with_match))
        result = validator.validate()

        # Should not have title_mismatch errors
        title_errors = [e for e in result.errors if e.error_code == "title_mismatch"]
        assert len(title_errors) == 0

    def test_validator_passes_with_auto_generated_title(self, temp_manuscript_with_auto_title):
        """Test that validator doesn't flag auto-generated titles as mismatch."""
        validator = SyntaxValidator(str(temp_manuscript_with_auto_title))
        result = validator.validate()

        # Should not have title_mismatch errors (auto-generated titles are ok)
        title_errors = [e for e in result.errors if e.error_code == "title_mismatch"]
        assert len(title_errors) == 0

    def test_validator_handles_missing_config(self):
        """Test validator handles missing config gracefully."""
        with tempfile.TemporaryDirectory() as tmpdir:
            manuscript_path = Path(tmpdir)

            # Create main without config
            main_path = manuscript_path / "01_MAIN.md"
            with open(main_path, "w") as f:
                f.write("# Title\n\n## Introduction\n")

            validator = SyntaxValidator(str(manuscript_path))
            result = validator.validate()

            # Should not crash or have title_mismatch errors
            title_errors = [e for e in result.errors if e.error_code == "title_mismatch"]
            assert len(title_errors) == 0

    def test_validator_handles_missing_main(self):
        """Test validator handles missing main file gracefully."""
        with tempfile.TemporaryDirectory() as tmpdir:
            manuscript_path = Path(tmpdir)

            # Create config without main
            config_path = manuscript_path / "00_CONFIG.yml"
            with open(config_path, "w") as f:
                yaml.dump({"title": "Title"}, f)

            validator = SyntaxValidator(str(manuscript_path))
            result = validator.validate()

            # Should not crash or have title_mismatch errors
            title_errors = [e for e in result.errors if e.error_code == "title_mismatch"]
            assert len(title_errors) == 0


class TestValidationErrorDetails:
    """Tests for validation error message details."""

    def test_error_includes_line_number(self, temp_manuscript_with_mismatch):
        """Test that error includes line number of title in main."""
        validator = SyntaxValidator(str(temp_manuscript_with_mismatch))
        result = validator.validate()

        title_errors = [e for e in result.errors if e.error_code == "title_mismatch"]
        assert len(title_errors) == 1

        error = title_errors[0]
        assert error.line_number is not None
        assert error.line_number == 1  # Title is on first line

    def test_error_includes_helpful_suggestion(self, temp_manuscript_with_mismatch):
        """Test that error includes helpful suggestion."""
        validator = SyntaxValidator(str(temp_manuscript_with_mismatch))
        result = validator.validate()

        title_errors = [e for e in result.errors if e.error_code == "title_mismatch"]
        assert len(title_errors) == 1

        error = title_errors[0]
        assert error.suggestion is not None
        # Should mention updating one or the other
        assert "00_CONFIG.yml" in error.suggestion or "01_MAIN.md" in error.suggestion



"""Tests for comment processing functionality.

This module tests the critical comment filtering functionality that ensures
commented content is never processed as active content. This is essential
for both security and correctness.
"""

from rxiv_maker.converters.comment_processor import (
    _find_comment_start,
    _find_latex_comment_start,
    filter_latex_comments,
    filter_python_comments,
    preprocess_comments,
    remove_html_comments,
)


class TestHtmlCommentRemoval:
    """Test HTML comment removal functionality."""

    def test_remove_simple_html_comment(self):
        """Test removal of simple HTML comments."""
        input_text = "Before <!-- This is a comment --> After"
        expected = "Before  After"
        result = remove_html_comments(input_text)
        assert result == expected

    def test_remove_multiline_html_comment(self):
        """Test removal of multiline HTML comments."""
        input_text = """Before
<!-- This is a
multiline comment
spanning several lines -->
After"""
        expected = """Before

After"""
        result = remove_html_comments(input_text)
        assert result == expected

    def test_remove_comment_with_table(self):
        """Test removal of HTML comment containing a table (the critical case)."""
        # Create the input without escaping issues
        input_text = (
            "Before table\n\n"
            "<!--| Name | Description | Reference |\n"
            "|-------|----------|--------|\n"
            "| JE2 | Derivative of community-acquired MRSA | @fey_genetic_2013 |\n"
            "| BCBMS14 | JE2 stuff containing stuff-2| %@reed_crispri-based_2024 |\n"
            "| LCML1 | JE2 stuff containing psg-0001_dnaA | @reed_crispri-based_2024 | -->\n\n"
            "After table"
        )
        expected = "Before table\n\n\n\nAfter table"
        result = remove_html_comments(input_text)
        assert result == expected
        # Ensure no table markdown syntax remains
        assert "|" not in result
        assert "@fey_genetic_2013" not in result
        assert "@reed_crispri-based_2024" not in result

    def test_remove_comment_with_citations(self):
        """Test that citations inside comments are removed."""
        input_text = """Regular text
<!-- This paper by @smith2023 and @jones2024 should not be processed -->
More text"""
        expected = """Regular text

More text"""
        result = remove_html_comments(input_text)
        assert result == expected
        assert "@smith2023" not in result
        assert "@jones2024" not in result

    def test_remove_comment_with_executable_blocks(self):
        """Test that executable blocks inside comments are removed."""
        input_text = """Before
<!--
{{py:exec
dangerous_code = "rm -rf /"
os.system(dangerous_code)
}}
The result is {{py:get dangerous_code}}
-->
After"""
        expected = """Before

After"""
        result = remove_html_comments(input_text)
        assert result == expected
        assert "py:exec" not in result
        assert "dangerous_code" not in result

    def test_remove_comment_with_tex_blocks(self):
        """Test that TeX blocks inside comments are removed."""
        input_text = """Before
<!-- {{tex: \\dangerous{command}}} -->
After"""
        expected = """Before

After"""
        result = remove_html_comments(input_text)
        assert result == expected
        assert "tex:" not in result
        assert "dangerous" not in result

    def test_multiple_comments(self):
        """Test removal of multiple HTML comments."""
        input_text = """Start
<!-- First comment -->
Middle
<!-- Second comment with @citation -->
End"""
        expected = """Start

Middle

End"""
        result = remove_html_comments(input_text)
        assert result == expected
        assert "comment" not in result
        assert "@citation" not in result

    def test_nested_comments(self):
        """Test handling of nested comment-like syntax."""
        input_text = """Before
<!-- Outer comment <!-- inner text --> still commented -->
After"""
        # Our algorithm removes comments from first <!-- to first -->
        # This is acceptable behavior for security purposes
        result = remove_html_comments(input_text)
        assert "inner text" not in result
        # Note: "still commented -->" will remain due to our simple parsing
        # This is acceptable as perfect nested comment parsing is complex
        # and the security goal is achieved

    def test_malformed_comments(self):
        """Test handling of malformed comments."""
        input_text = """Before
<!-- Unclosed comment
Regular text after unclosed comment"""
        # Should leave malformed comments as-is (don't break the document)
        result = remove_html_comments(input_text)
        assert "Unclosed comment" in result  # Malformed comment preserved

    def test_empty_comments(self):
        """Test removal of empty comments."""
        input_text = "Before <!-- --> After"
        expected = "Before  After"
        result = remove_html_comments(input_text)
        assert result == expected

    def test_comments_with_special_characters(self):
        """Test comments containing special regex characters."""
        input_text = "Before <!-- Comment with $pecial ^characters* and [brackets] --> After"
        expected = "Before  After"
        result = remove_html_comments(input_text)
        assert result == expected


class TestPythonCommentFiltering:
    """Test Python comment filtering within executable blocks."""

    def test_filter_full_line_comment(self):
        """Test filtering of full-line Python comments."""
        code = """x = 5
# This is a comment
y = 10"""
        expected = """x = 5

y = 10"""
        result = filter_python_comments(code)
        assert result == expected

    def test_filter_inline_comment(self):
        """Test filtering of inline Python comments."""
        code = "x = 5  # This is an inline comment"
        expected = "x = 5"
        result = filter_python_comments(code)
        assert result == expected

    def test_preserve_hash_in_strings(self):
        """Test that # inside strings is preserved."""
        code = """message = "Use #hashtags in social media"  # This is a comment"""
        expected = '''message = "Use #hashtags in social media"'''
        result = filter_python_comments(code)
        assert result == expected
        assert "#hashtags" in result
        assert "This is a comment" not in result

    def test_complex_python_with_comments(self):
        """Test filtering comments in complex Python code."""
        code = """# Initialize variables
import math  # Math library
x = 10  # Set x value
# Calculate result
result = math.sqrt(x)  # Square root
print(f"Result: {result}")  # Display result"""

        result = filter_python_comments(code)

        # Should preserve the actual code
        assert "import math" in result
        assert "x = 10" in result
        assert "result = math.sqrt(x)" in result
        assert 'print(f"Result: {result}")' in result

        # Should remove all comments
        assert "Initialize variables" not in result
        assert "Math library" not in result
        assert "Set x value" not in result
        assert "Calculate result" not in result
        assert "Square root" not in result
        assert "Display result" not in result

    def test_preserve_line_numbers(self):
        """Test that line numbers are preserved for error reporting."""
        code = """line1 = 1
# comment on line 2
line3 = 3
# comment on line 4
line5 = 5"""

        result = filter_python_comments(code)
        lines = result.split("\n")

        # Should have 5 lines (same as input)
        assert len(lines) == 5
        assert lines[0] == "line1 = 1"
        assert lines[1] == ""  # Comment replaced with empty line
        assert lines[2] == "line3 = 3"
        assert lines[3] == ""  # Comment replaced with empty line
        assert lines[4] == "line5 = 5"

    def test_hash_in_various_string_contexts(self):
        """Test # in different string contexts."""
        code = """
single = 'Price: $10 #1 item'  # Comment here
double = "Hashtag: #python"  # Another comment
multiline = '''
This has #hashtags
and # symbols
'''  # Final comment
"""
        result = filter_python_comments(code)

        # Should preserve all the # inside strings
        assert "#1 item" in result
        assert "#python" in result
        # Note: multiline strings are tricky - the current implementation
        # processes line by line, so # inside multiline strings may not be preserved
        # This is acceptable for the security goal of preventing dangerous code execution

        # Should remove all actual comments
        assert "Comment here" not in result
        assert "Another comment" not in result
        # Note: The final comment is tricky because it's after the multiline string close


class TestLatexCommentFiltering:
    """Test LaTeX comment filtering within TeX blocks."""

    def test_filter_full_line_latex_comment(self):
        """Test filtering of full-line LaTeX comments."""
        code = r"""\textbf{Bold}
% This is a comment
\textit{Italic}"""
        expected = r"""\textbf{Bold}

\textit{Italic}"""
        result = filter_latex_comments(code)
        assert result == expected

    def test_filter_inline_latex_comment(self):
        """Test filtering of inline LaTeX comments."""
        code = r"\textbf{Bold} % This is an inline comment"
        expected = r"\textbf{Bold}"
        result = filter_latex_comments(code)
        assert result == expected

    def test_preserve_escaped_percent(self):
        """Test that escaped % (\\%) is preserved."""
        code = r"\text{Price: \$100 \& tax: 5\% (total: \$105)} % This is a comment"
        expected = r"\text{Price: \$100 \& tax: 5\% (total: \$105)}"
        result = filter_latex_comments(code)
        assert result == expected
        assert r"5\%" in result  # Escaped % preserved
        assert "This is a comment" not in result

    def test_complex_latex_with_comments(self):
        """Test filtering comments in complex LaTeX code."""
        code = r"""% Document setup
\documentclass{article} % Article class
\usepackage{amsmath}    % Math package
% Begin document
\begin{document}
\title{My Title} % Document title
\author{Author}  % Author name
\end{document}   % End document"""

        result = filter_latex_comments(code)

        # Should preserve LaTeX commands
        assert r"\documentclass{article}" in result
        assert r"\usepackage{amsmath}" in result
        assert r"\begin{document}" in result
        assert r"\title{My Title}" in result
        assert r"\author{Author}" in result
        assert r"\end{document}" in result

        # Should remove all comments
        assert "Document setup" not in result
        assert "Article class" not in result
        assert "Math package" not in result
        assert "Begin document" not in result
        assert "Document title" not in result
        assert "Author name" not in result
        assert "End document" not in result

    def test_multiple_escaped_percent(self):
        """Test handling of multiple backslashes before %."""
        code = r"""
\text{One backslash: \%}  % Comment 1
\text{Two backslashes: \\%} % Comment 2
\text{Three backslashes: \\\%} % Comment 3
"""
        result = filter_latex_comments(code)

        # \% should be preserved (escaped)
        assert r"\%" in result
        # \\% should have the % removed (not escaped)
        assert r"Two backslashes: \\" in result
        # \\\% should be preserved (escaped)
        assert r"\\\%" in result

        # All comments should be removed
        assert "Comment 1" not in result
        assert "Comment 2" not in result
        assert "Comment 3" not in result


class TestCommentHelperFunctions:
    """Test helper functions for comment detection."""

    def test_find_python_comment_start(self):
        """Test Python comment start detection."""
        # Full line comment
        assert _find_comment_start("# Full line comment") == 0
        assert _find_comment_start("   # Indented comment") == 0

        # Inline comment
        assert _find_comment_start("x = 5  # Inline comment") == 7

        # No comment
        assert _find_comment_start("x = 5") == -1
        assert _find_comment_start("") == -1

        # Hash in string
        assert _find_comment_start('message = "Use #hashtags"') == -1
        assert _find_comment_start("url = 'http://example.com#section'") == -1

        # Hash in string with comment after - need to account for exact position
        test_line = 'msg = "#tag"  # comment'
        expected_pos = test_line.find("  # comment") + 2  # Position of the # in comment
        assert _find_comment_start(test_line) == expected_pos

    def test_find_latex_comment_start(self):
        """Test LaTeX comment start detection."""
        # Full line comment
        assert _find_latex_comment_start("% Full line comment") == 0
        assert _find_latex_comment_start("   % Indented comment") == 0

        # Inline comment
        assert _find_latex_comment_start(r"\textbf{Bold} % comment") == 14

        # No comment
        assert _find_latex_comment_start(r"\textbf{Bold}") == -1
        assert _find_latex_comment_start("") == -1

        # Escaped percent
        assert _find_latex_comment_start(r"\text{5\% tax}") == -1

        # Escaped percent with comment after
        assert _find_latex_comment_start(r"\text{5\% tax} % comment") == 15


class TestPreprocessComments:
    """Test the main preprocessing function."""

    def test_preprocess_removes_html_comments(self):
        """Test that preprocess_comments removes HTML comments."""
        input_text = """Regular text
<!-- This should be removed -->
More text"""
        expected = """Regular text

More text"""
        result = preprocess_comments(input_text)
        assert result == expected

    def test_preprocess_with_mixed_content(self):
        """Test preprocessing with tables, citations, and executable blocks in comments."""
        input_text = """# Document Title

Regular content here.

<!--
| Commented Table | Should Not | Appear |
|-----------------|------------|---------|
| Row 1           | @cite1     | Data    |

{{py:exec
# This dangerous code should never execute
import os
os.system("rm -rf /")
}}

{{tex: \\dangerous{command}}}
-->

More regular content.
"""
        result = preprocess_comments(input_text)

        # Should preserve regular content
        assert "# Document Title" in result
        assert "Regular content here." in result
        assert "More regular content." in result

        # Should remove all commented content
        assert "Commented Table" not in result
        assert "@cite1" not in result
        assert "py:exec" not in result
        assert "os.system" not in result
        assert "dangerous{command}" not in result


class TestIntegrationScenarios:
    """Test integration scenarios mimicking real-world usage."""

    def test_security_scenario_dangerous_code_commented(self):
        """Test that dangerous code in comments never gets processed."""
        input_text = """
# Safe Document

This is safe content.

<!--
{{py:exec
import subprocess
subprocess.run(['rm', '-rf', '/'], check=True)
}}

The result is: {{py:get malicious_result}}

| Malicious | Table | With Citations |
|-----------|-------|----------------|
| Evil      | @bad_paper | {{py:get secret_data}} |
-->

End of document.
"""
        result = preprocess_comments(input_text)

        # Should preserve safe content
        assert "# Safe Document" in result
        assert "This is safe content." in result
        assert "End of document." in result

        # Should remove ALL dangerous content
        assert "py:exec" not in result
        assert "subprocess" not in result
        assert "rm -rf" not in result
        assert "py:get" not in result
        assert "malicious_result" not in result
        assert "secret_data" not in result
        assert "@bad_paper" not in result
        assert "Malicious" not in result

    def test_complex_nested_content_in_comments(self):
        """Test complex nested content inside comments."""
        input_text = """
Regular text

<!--
## Commented Section

This section contains:

- Lists with @citations
- Tables with executable content:

| Name | Code | Result |
|------|------|--------|
| Test | {{py:exec x=1}} | {{py:get x}} |

And some {{tex: \\LaTeX{} commands}}.

### Subsection

More content that should be ignored.
-->

Final text
"""
        result = preprocess_comments(input_text)

        # Only regular content should remain
        assert "Regular text" in result
        assert "Final text" in result

        # All commented content should be gone
        assert "Commented Section" not in result
        assert "Lists with" not in result
        assert "@citations" not in result
        assert "py:exec" not in result
        assert "LaTeX{}" not in result
        assert "Subsection" not in result
        assert "should be ignored" not in result



"""Cache utilities for rxiv-maker.

Provides manuscript-local cache directory management using .rxiv_cache.
"""

from pathlib import Path


def find_manuscript_directory(start_path: Path | None = None, max_depth: int = 5) -> Path | None:
    """Find the manuscript directory by locating 00_CONFIG.yml file.

    Walks up the directory tree from the starting path to find a directory
    containing 00_CONFIG.yml, which indicates a manuscript root.

    Args:
        start_path: Path to start searching from (defaults to current directory)
        max_depth: Maximum depth to search up the directory tree

    Returns:
        Path to manuscript directory if found, None otherwise

    Examples:
        >>> find_manuscript_directory()
        PosixPath('/path/to/manuscript')  # if 00_CONFIG.yml found

        >>> find_manuscript_directory(Path('/path/to/manuscript/subdir'))
        PosixPath('/path/to/manuscript')  # walks up to find config
    """
    current_path = (start_path or Path.cwd()).resolve()

    for _i in range(max_depth):
        config_file = current_path / "00_CONFIG.yml"
        if config_file.exists() and config_file.is_file():
            return current_path

        # Move up one directory
        parent = current_path.parent
        if parent == current_path:
            # Reached filesystem root
            break
        current_path = parent

    return None


def get_manuscript_cache_dir(subfolder: str | None = None, manuscript_dir: Path | None = None) -> Path:
    """Get the manuscript-local cache directory (.rxiv_cache in manuscript directory).

    Args:
        subfolder: Optional subfolder within the cache directory
        manuscript_dir: Manuscript directory (if None, auto-detected)

    Returns:
        Path to manuscript-local cache directory

    Raises:
        RuntimeError: If no manuscript directory is found

    Examples:
        >>> get_manuscript_cache_dir()
        PosixPath('/path/to/manuscript/.rxiv_cache')

        >>> get_manuscript_cache_dir("doi")
        PosixPath('/path/to/manuscript/.rxiv_cache/doi')
    """
    if manuscript_dir is None:
        manuscript_dir = find_manuscript_directory()

    if manuscript_dir is None:
        raise RuntimeError(
            "Cannot find manuscript directory. Make sure you're in a directory containing 00_CONFIG.yml "
            "or one of its subdirectories."
        )

    cache_dir = manuscript_dir / ".rxiv_cache"

    if subfolder:
        cache_dir = cache_dir / subfolder

    # Ensure directory exists
    try:
        cache_dir.mkdir(parents=True, exist_ok=True)
    except (OSError, PermissionError) as e:
        raise RuntimeError(f"Cannot create cache directory {cache_dir}: {e}") from e

    return cache_dir


def get_manuscript_name(manuscript_dir: Path | None = None) -> str | None:
    """Get the manuscript name from the manuscript directory.

    Args:
        manuscript_dir: Manuscript directory (if None, auto-detected)

    Returns:
        Manuscript directory name if found, None otherwise

    Examples:
        >>> get_manuscript_name()
        'MANUSCRIPT'  # if in /path/to/MANUSCRIPT directory
    """
    if manuscript_dir is None:
        manuscript_dir = find_manuscript_directory()

    if manuscript_dir is None:
        return None

    return manuscript_dir.name


def is_in_manuscript_directory() -> bool:
    """Check if current working directory is within a manuscript directory.

    Returns:
        True if in a manuscript directory, False otherwise

    Examples:
        >>> is_in_manuscript_directory()
        True  # if 00_CONFIG.yml found in current dir or parent dirs
    """
    return find_manuscript_directory() is not None



"""Bibliography file parsing utilities.

This module provides utilities for parsing BibTeX files and extracting entry information.
Used by CLI commands to provide structured bibliography data.
"""

import re
from dataclasses import dataclass
from pathlib import Path
from typing import Any


@dataclass
class BibEntry:
    """Represents a parsed bibliography entry."""

    key: str  # Citation key (e.g., "smith2020")
    entry_type: str  # Entry type (e.g., "article", "book")
    fields: dict[str, str]  # Entry fields (title, author, year, etc.)
    raw: str  # Raw BibTeX entry


def parse_bib_file(bib_path: Path) -> list[BibEntry]:
    """Parse a BibTeX file and extract all entries.

    Args:
        bib_path: Path to the .bib file

    Returns:
        List of parsed bibliography entries

    Raises:
        FileNotFoundError: If the bibliography file doesn't exist
        ValueError: If the file cannot be parsed
    """
    if not bib_path.exists():
        raise FileNotFoundError(f"Bibliography file not found: {bib_path}")

    try:
        content = bib_path.read_text(encoding="utf-8")
    except Exception as e:
        raise ValueError(f"Could not read bibliography file: {e}") from e

    return parse_bib_content(content)


def parse_bib_content(content: str) -> list[BibEntry]:
    """Parse BibTeX content and extract all entries.

    Args:
        content: BibTeX file content

    Returns:
        List of parsed bibliography entries
    """
    entries = []

    # Pattern to match complete bib entries: @type{key, ...fields...}
    # We need to manually track braces since BibTeX entries can contain nested braces
    entry_start_pattern = re.compile(r"@(\w+)\s*\{\s*([^,\s}]+)\s*,", re.MULTILINE)

    for match in entry_start_pattern.finditer(content):
        entry_type = match.group(1).lower()
        key = match.group(2).strip()

        # Find the matching closing brace for this entry
        start_pos = match.end()
        brace_count = 1  # We've seen the opening brace
        end_pos = start_pos

        for i, char in enumerate(content[start_pos:], start=start_pos):
            if char == "{":
                brace_count += 1
            elif char == "}":
                brace_count -= 1
                if brace_count == 0:
                    end_pos = i
                    break

        # Extract fields content (between comma after key and closing brace)
        fields_content = content[start_pos:end_pos].strip()

        # Parse individual fields
        fields = _parse_fields(fields_content)

        # Extract raw entry (from @ to closing brace)
        raw_text = content[match.start() : end_pos + 1]

        entry = BibEntry(key=key, entry_type=entry_type, fields=fields, raw=raw_text)
        entries.append(entry)

    return entries


def _parse_fields(fields_content: str) -> dict[str, str]:
    """Parse fields from a bibliography entry.

    Args:
        fields_content: The content between the comma after the key and the closing brace

    Returns:
        Dictionary of field names to values
    """
    fields = {}

    # Pattern to match:
    # - field = {value}  (braced values)
    # - field = "value"  (quoted values)
    # - field = value    (bare values - numbers, single words)
    # Handles multi-line values and nested braces

    # Helper function to extract braced field values with proper nesting
    def extract_braced_value(text: str, start_pos: int) -> tuple[str, int]:
        """Extract value from braced field, handling nested braces.

        Returns: (value, end_position)
        """
        if start_pos >= len(text) or text[start_pos] != "{":
            return ("", start_pos)

        brace_count = 0
        value_start = start_pos + 1
        i = start_pos

        while i < len(text):
            if text[i] == "{":
                brace_count += 1
            elif text[i] == "}":
                brace_count -= 1
                if brace_count == 0:
                    return (text[value_start:i], i + 1)
            i += 1

        return (text[value_start:], len(text))

    # Simpler pattern that finds field names and equals signs
    field_start_pattern = re.compile(
        r'(\w+)\s*=\s*(["{])',  # Field name, =, and opening delimiter
        re.MULTILINE,
    )

    for field_match in field_start_pattern.finditer(fields_content):
        field_name = field_match.group(1).strip().lower()
        delimiter = field_match.group(2)
        value_start = field_match.end() - 1  # Back up to the delimiter

        if delimiter == "{":
            # Extract braced value with proper nesting
            field_value, _end_pos = extract_braced_value(fields_content, value_start)
        elif delimiter == '"':
            # Extract quoted value
            quote_end = fields_content.find('"', value_start + 1)
            if quote_end != -1:
                field_value = fields_content[value_start + 1 : quote_end]
            else:
                field_value = fields_content[value_start + 1 :]
        else:
            continue

        # Clean up the value
        field_value = field_value.strip()
        field_value = " ".join(field_value.split())

        if field_value:
            fields[field_name] = field_value

    # Also handle bare values (numbers, etc.) not already processed
    bare_pattern = re.compile(r'(\w+)\s*=\s*([^,}\s"{][^,}]*)', re.MULTILINE)

    for field_match in bare_pattern.finditer(fields_content):
        field_name = field_match.group(1).strip().lower()

        # Skip if already processed
        if field_name in fields:
            continue

        field_value = field_match.group(2).strip().rstrip(",").strip()
        field_value = " ".join(field_value.split())

        if field_value:
            fields[field_name] = field_value

    return fields


def entry_to_dict(entry: BibEntry, include_raw: bool = False) -> dict[str, Any]:
    """Convert a BibEntry to a dictionary for JSON serialization.

    Args:
        entry: The bibliography entry
        include_raw: Whether to include the raw BibTeX entry

    Returns:
        Dictionary representation of the entry
    """
    result = {"key": entry.key, "type": entry.entry_type, **entry.fields}

    if include_raw:
        result["raw"] = entry.raw

    return result


def format_author_list(author_string: str) -> list[str]:
    """Format author string into a list of individual authors.

    Args:
        author_string: The author field from a BibTeX entry (e.g., "Smith, J. and Doe, J.")

    Returns:
        List of author names
    """
    if not author_string:
        return []

    # Split on 'and' (case-insensitive, with word boundaries)
    authors = re.split(r"\s+and\s+", author_string, flags=re.IGNORECASE)

    # Clean up each author name
    return [author.strip() for author in authors if author.strip()]



"""Integration tests for the complete comment system functionality.

This module tests the end-to-end functionality of the comment filtering system
across all three types of comments: HTML, Python, and LaTeX. It ensures that
the security fixes work correctly in the full processing pipeline.
"""

from rxiv_maker.converters.custom_command_processor import process_custom_commands
from rxiv_maker.converters.md2tex import convert_markdown_to_latex


class TestCommentSystemIntegration:
    """Test end-to-end comment system integration."""

    def test_html_comment_filtering_in_full_pipeline(self):
        """Test that HTML comments are filtered out in the complete processing pipeline."""
        # Create markdown with HTML comments containing dangerous content
        markdown_content = """
# Document Title

Regular content here.

<!--
| Dangerous | Table | Should Not Appear |
|-----------|-------|-------------------|
| Row 1     | @malicious_citation | Data |

{{py:exec
import os
os.system("rm -rf /")
}}

{{tex: \\dangerouscommand{malicious}}}
-->

More regular content after the comment.
"""

        # Process through the full pipeline
        result = convert_markdown_to_latex(markdown_content)

        # Should preserve safe content
        assert "Document Title" in result
        assert "Regular content here." in result
        assert "More regular content after the comment." in result

        # Should remove ALL dangerous commented content
        assert "Dangerous" not in result
        assert "Table" not in result
        assert "Should Not Appear" not in result
        assert "@malicious_citation" not in result
        assert "py:exec" not in result
        assert "os.system" not in result
        assert "rm -rf" not in result
        assert "dangerouscommand" not in result
        assert "malicious" not in result

    def test_python_comment_filtering_in_executable_blocks(self):
        """Test that Python comments are filtered in executable blocks."""
        markdown_content = """
{{py:exec
# This is a dangerous comment that should not execute
import math
x = 5  # This comment should also be filtered
# y = dangerous_function()
result = math.sqrt(x)
}}

The result is: {{py:get result}}
"""

        # Process the custom commands
        result = process_custom_commands(markdown_content)

        # Should execute the valid Python code
        assert "2.236" in result or "2.23606" in result  # sqrt(5) ≈ 2.236

        # Should not contain any comments in the output or processing
        # Note: We can't easily test that comments weren't executed since they're filtered,
        # but we can verify the result is mathematically correct

    def test_latex_comment_filtering_in_tex_blocks(self):
        """Test that LaTeX comments are filtered in TeX injection blocks."""
        markdown_content = """
{{tex:
% This comment should be filtered
\\textbf{Bold Text} % Another comment
% \\dangerouscommand{should not appear}
\\textit{Italic Text}
}}
"""

        # Process the custom commands
        result = process_custom_commands(markdown_content)

        # Should preserve LaTeX commands
        assert "\\textbf{Bold Text}" in result
        assert "\\textit{Italic Text}" in result

        # Should filter out comments
        assert "% This comment should be filtered" not in result
        assert "% Another comment" not in result
        assert "dangerouscommand" not in result

    def test_mixed_comment_types_in_same_document(self):
        """Test handling of all three comment types in the same document."""
        markdown_content = """
# Test Document

Regular content.

<!-- HTML comment with table:
| Should | Not | Appear |
|--------|-----|--------|
| Row    | @bad_cite | Data |
-->

{{py:exec
# Python comment: should not execute
import random
# seed = 12345  # This should not set seed
random.seed(42)
value = random.randint(1, 100)
}}

Result: {{py:get value}}

{{tex:
% LaTeX comment: should be filtered
\\textbf{Bold} % Inline comment
% \\badcommand{dangerous}
\\textit{Italic}
}}

Final content.
"""

        # Process through the full pipeline
        result = convert_markdown_to_latex(markdown_content)

        # Should preserve good content
        assert "Test Document" in result
        assert "Regular content." in result
        assert "Final content." in result
        assert "\\textbf{Bold}" in result
        assert "\\textit{Italic}" in result

        # Should remove HTML commented content
        assert "Should" not in result or "Should" in "Test Document"  # Avoid false positive
        assert "@bad_cite" not in result

        # Should execute Python (comments filtered)
        assert "Result:" in result
        # Should have deterministic result due to seed(42)

        # Should filter LaTeX comments
        assert "% LaTeX comment" not in result
        assert "badcommand" not in result

    def test_comment_security_edge_cases(self):
        """Test edge cases for comment security."""
        markdown_content = """
<!-- Tricky case: HTML comment with nested content
{{py:exec
import subprocess
subprocess.run(["rm", "-rf", "/"], check=True)
}}

And a table:
| Evil | Table | Here |
|------|-------|------|
| @dangerous | {{py:get malicious}} | Bad |

{{tex: \\evilcommand{destroy everything}}}
-->

This should remain: {{py:exec
result = "safe"
}}

Safe result: {{py:get result}}
"""

        result = convert_markdown_to_latex(markdown_content)

        # Should preserve safe content
        assert "This should remain:" in result
        assert "safe" in result

        # Should remove ALL dangerous commented content
        assert "subprocess" not in result
        assert "rm -rf" not in result
        assert "Evil" not in result
        assert "@dangerous" not in result
        assert "evilcommand" not in result
        assert "destroy everything" not in result

    def test_comment_system_preserves_functionality(self):
        """Test that comment filtering doesn't break normal functionality."""
        markdown_content = """
# Normal Document

This is regular markdown content.

{{py:exec
import math
pi_value = math.pi
squared = 2 ** 2
}}

Mathematical constants:
- Pi: {{py:get pi_value}}
- Two squared: {{py:get squared}}

{{tex: \\textbf{Bold text} and \\textit{italic text}}}

| Normal | Table | Works |
|--------|-------|-------|
| Row 1  | Data  | Here  |

Citations still work: @example2023

Final paragraph.
"""

        result = convert_markdown_to_latex(markdown_content)

        # Should preserve all normal functionality
        assert "Normal Document" in result
        assert "regular markdown content" in result
        assert "Mathematical constants" in result
        assert "3.14159" in result  # Pi value
        assert "4" in result  # 2^2 = 4
        assert "\\textbf{Bold text}" in result
        assert "\\textit{italic text}" in result
        assert "Normal" in result and "Table" in result  # Table processing
        assert "\\cite{example2023}" in result  # Citations
        assert "Final paragraph" in result

    def test_malformed_html_comments_handled_safely(self):
        """Test that malformed HTML comments don't break processing."""
        markdown_content = """
Regular content.

<!-- Unclosed comment with dangerous content
{{py:exec
dangerous_code = "should not execute"
}}

More content that might be affected.
"""

        # Should process without crashing
        result = convert_markdown_to_latex(markdown_content)

        # Should preserve safe content
        assert "Regular content." in result
        assert "More content that might be affected." in result

        # The malformed comment content should remain (as it's not properly closed)
        # This is acceptable behavior - perfect parsing is complex

    def test_nested_content_in_comments_filtered(self):
        """Test that complex nested content inside comments is filtered."""
        markdown_content = """
Before comment.

<!--
## Commented Section

This section contains:

- Lists with @malicious_citations
- Complex tables:

| Name | Code | Danger Level |
|------|------|--------------|
| Test | {{py:exec malicious=True}} | {{py:get malicious}} |

### Subsection

More dangerous content: {{tex: \\input{/etc/passwd}}}

And some math: $E = mc^2$
-->

After comment.
"""

        result = convert_markdown_to_latex(markdown_content)

        # Should only have safe content
        assert "Before comment." in result
        assert "After comment." in result

        # Should remove all nested dangerous content
        assert "Commented Section" not in result
        assert "@malicious_citations" not in result
        assert "malicious=True" not in result
        assert "Danger Level" not in result
        assert "\\input{/etc/passwd}" not in result
        # Note: $E = mc^2$ might be tricky to test as it could appear elsewhere


class TestCommentSystemPerformance:
    """Test that comment filtering doesn't significantly impact performance."""

    def test_large_document_with_comments(self):
        """Test processing of large document with many comments."""
        # Create a moderately large document with various comment types
        sections = []
        for i in range(50):
            section = f"""
## Section {i}

Content for section {i}.

<!-- Comment {i} with table:
| Col1 | Col2 | Col3 |
|------|------|------|
| Data | @cite{i} | More |
-->

{{py:exec
# Comment in Python block {i}
section_num = {i}
}}

Section number: {{py:get section_num}}

{{tex: \\textbf{{Section {i}}} % Comment in LaTeX}}
"""
            sections.append(section)

        large_content = "\n".join(sections)

        # Should process without timeout or excessive memory usage
        result = convert_markdown_to_latex(large_content)

        # Should preserve good content
        assert "Section 0" in result
        assert "Section 49" in result

        # Should remove commented content (spot check)
        assert "@cite" not in result
        assert "Comment 25 with table" not in result


class TestCommentSystemRegression:
    """Test that comment system fixes don't break existing functionality."""

    def test_existing_markdown_features_still_work(self):
        """Test that standard markdown features continue to work."""
        markdown_content = """
# Main Title

## Subsection

**Bold text** and *italic text*.

- List item 1
- List item 2
  - Nested item

1. Numbered item
2. Another numbered item

`inline code` and:

```python
code_block = "should not be processed as command"
print(code_block)
```

[Link text](http://example.com)

![Figure caption](figure.png){#fig:example}

| Table | Headers |
|-------|---------|
| Data  | More    |

Mathematical formula: $E = mc^2$

> Block quote text

Final paragraph with @citation2023.
"""

        result = convert_markdown_to_latex(markdown_content)

        # Should process all standard markdown correctly
        assert "Main Title" in result
        assert "Subsection" in result
        assert "textbf{Bold text}" in result
        assert "textit{italic text}" in result
        assert "\\item List item 1" in result
        assert "\\item Numbered item" in result
        assert "texttt{inline code}" in result
        assert "begin{lstlisting}" in result or "begin{verbatim}" in result  # Code blocks
        assert "href{http://example.com}" in result
        assert "includegraphics" in result  # Figures
        assert "begin{table}" in result or "begin{tabular}" in result  # Tables
        assert "$E = mc^2$" in result  # Math
        assert "\\cite{citation2023}" in result  # Citations



"""Text formatting processors for markdown to LaTeX conversion.

This module handles basic text formatting including bold, italic, code,
headers, and special character escaping.
"""

import re

from .types import LatexContent, MarkdownContent

# Pre-compiled regex patterns for performance optimization
# Text formatting patterns
BOLD_PATTERN = re.compile(r"\*\*(.+?)\*\*")
ITALIC_PATTERN = re.compile(r"\*(.+?)\*")
UNDERLINE_PATTERN = re.compile(r"__([^_\n]*(?:_[^_\n]*)*?)__")

# Bold/italic patterns with constraints
BOLD_CONSTRAINED_PATTERN = re.compile(r"\*\*([^*]+)\*\*")
ITALIC_CONSTRAINED_PATTERN = re.compile(r"(?<!\*)\*([^*]+?)\*(?!\*)")

# Header patterns
H2_PATTERN = re.compile(r"^## (.+)$", re.MULTILINE)
H3_PATTERN = re.compile(r"^### (.+)$", re.MULTILINE)
H4_PATTERN = re.compile(r"^#### (.+)$", re.MULTILINE)

# Code block patterns
DOUBLE_BACKTICK_PATTERN = re.compile(r"``([^`]+)``")
SINGLE_BACKTICK_PATTERN = re.compile(r"`([^`]+)`")

# LaTeX command patterns
LATEX_COMMAND_PATTERN = re.compile(r"(\\[a-zA-Z]+\{[^}]*\})")
TEXTTT_PATTERN = re.compile(r"(\\texttt\{[^}]*\})")

# Special character patterns
PERCENT_PATTERN = re.compile(r"(?<!\\)(?<!^)%", re.MULTILINE)
CARET_PATTERN = re.compile(r"(?<!\$)(?<!\\\$)\^(?!\^)(?![^$]*\$)")

# File path patterns
PARENS_PATTERN = re.compile(r"\(([^)]+)\)")
UNDERSCORE_ID_PATTERN = re.compile(r"\b\d+_[A-Z_]+\b")

# LaTeX cleanup patterns
DOUBLE_BACKSLASH_SPACED_PATTERN = re.compile(r"\\textbackslash textbackslash\s+")
DOUBLE_BACKSLASH_PATTERN = re.compile(r"\\textbackslash textbackslash")

# Protected environment patterns (will be built dynamically)
_PROTECTED_ENV_PATTERN = None


def _get_protected_env_pattern():
    """Get or create the protected environment pattern."""
    global _PROTECTED_ENV_PATTERN
    if _PROTECTED_ENV_PATTERN is None:
        # Separate non-starred and starred environments for proper backreference matching
        base_environments = [
            "equation",
            "align",
            "gather",
            "multiline",
            "split",
            "verbatim",
            "lstlisting",
            "tabular",
            "tabularx",
            "longtable",
            "array",
            "minipage",
            "figure",
            "sfigure",
            "sidewaysfigure",
        ]

        # Create patterns for both non-starred and starred versions
        patterns = []
        patterns.append("\\\\texttt\\{[^}]*\\}")  # texttt blocks

        for env in base_environments:
            # Non-starred version
            patterns.append(f"\\\\begin\\{{{re.escape(env)}\\}}.*?\\\\end\\{{{re.escape(env)}\\}}")
            # Starred version
            patterns.append(f"\\\\begin\\{{{re.escape(env)}\\*\\}}.*?\\\\end\\{{{re.escape(env)}\\*\\}}")

        # Combine all patterns into one with proper grouping
        pattern = f"({'|'.join(patterns)})"
        _PROTECTED_ENV_PATTERN = re.compile(pattern, re.DOTALL)
    return _PROTECTED_ENV_PATTERN


def _apply_formatting_outside_protected_environments(
    text: MarkdownContent, formatting_pattern: re.Pattern[str], replacement: str, use_full_protection: bool = True
) -> LatexContent:
    r"""Generic function to apply text formatting outside protected environments.

    Args:
        text: Text to process
        formatting_pattern: Pre-compiled regex pattern for the formatting
        replacement: Replacement string (e.g., r"\\textbf{\1}")
        use_full_protection: Whether to use full environment protection (True) or just texttt (False)

    Returns:
        Text with formatting applied outside protected environments
    """
    if use_full_protection:
        # Split by both \texttt{} blocks and LaTeX environments
        protection_pattern = _get_protected_env_pattern()
        parts = protection_pattern.split(text)

        result: list[str] = []
        for part in parts:
            if part is None:
                continue
            if part.startswith("\\texttt{") or part.startswith("\\begin{"):
                # This is a protected block, don't process it
                result.append(part)
            else:
                # This is regular text, apply formatting
                part = formatting_pattern.sub(replacement, part)
                result.append(part)
        return "".join(result)
    else:
        # Split by \texttt{} blocks only
        parts = TEXTTT_PATTERN.split(text)
        result: list[str] = []

        for part in parts:
            if part.startswith("\\texttt{"):
                # This is a texttt block, don't process it
                result.append(part)
            else:
                # This is regular text, apply formatting
                part = formatting_pattern.sub(replacement, part)
                result.append(part)
        return "".join(result)


def convert_subscript_superscript_to_latex(text: LatexContent) -> LatexContent:
    r"""Convert subscript and superscript markdown syntax to LaTeX.

    Avoids converting inside LaTeX commands like \\texttt{}.

    Args:
        text: Text content that may contain subscript and superscript

    Returns:
        LaTeX formatted text with subscript/superscript converted
    """

    # Helper function to avoid replacing inside LaTeX commands
    def replace_outside_commands(pattern, replacement, text):
        """Replace pattern with replacement, but not inside LaTeX commands or math mode."""
        # Combine all protection patterns into a single regex to avoid
        # sequential processing issues where one pattern affects another
        combined_pattern = (
            r"(\\texttt\{[^}]*\})|"  # \texttt{...}
            r"(\\text\{[^}]*\})|"  # \text{...}
            r"(\$[^$]*\$)|"  # Inline math $...$
            r"(\$\$.*?\$\$)|"  # Display math $$...$$
            r"(\\begin\{equation\}.*?\\end\{equation\})"  # equation environments
        )

        # Split by the combined pattern - protected parts will be in groups
        parts = re.split(combined_pattern, text, flags=re.DOTALL)
        result = []

        for part in parts:
            if part is None or part == "":
                continue

            # Check if this part matches any of our protection patterns
            is_protected = (
                part.startswith("\\texttt{")
                or part.startswith("\\text{")
                or (part.startswith("$") and not part.startswith("$$"))
                or part.startswith("$$")
                or part.startswith("\\begin{equation}")
            )

            if not is_protected:
                # Only apply replacement to unprotected parts
                part = re.sub(pattern, replacement, part)

            result.append(part)

        return "".join(result)

    # Convert simple subscript and superscript using markdown-style syntax
    # H~2~O becomes H\textsubscript{2}O
    text = replace_outside_commands(r"~([^~\s]+)~", r"\\textsubscript{\1}", text)
    # E=mc^2^ becomes E=mc\textsuperscript{2}
    text = replace_outside_commands(r"\^([^\^\s]+)\^", r"\\textsuperscript{\1}", text)

    return text


def convert_text_formatting_to_latex(text: MarkdownContent) -> LatexContent:
    """Convert markdown text formatting to LaTeX.

    Args:
        text: Markdown text with formatting

    Returns:
        LaTeX formatted text
    """
    # Convert bold and italic using pre-compiled patterns
    text = BOLD_PATTERN.sub(r"\\textbf{\1}", text)
    text = ITALIC_PATTERN.sub(r"\\textit{\1}", text)

    # Convert underlined text (single line only, don't match across line breaks)
    text = UNDERLINE_PATTERN.sub(r"\\underline{\1}", text)

    # Convert subscript and superscript
    text = convert_subscript_superscript_to_latex(text)

    # Note: Code conversion is handled by process_code_spans function
    # to properly support line breaking for long code spans

    return text


def convert_headers_to_latex(text: MarkdownContent) -> LatexContent:
    """Convert markdown headers to LaTeX sections.

    Args:
        text: Markdown text with headers

    Returns:
        LaTeX text with section commands
    """
    text = H2_PATTERN.sub(r"\\section{\1}", text)
    text = H3_PATTERN.sub(r"\\subsection{\1}", text)
    text = H4_PATTERN.sub(r"\\subsubsection{\1}", text)

    return text


def process_code_spans(text: MarkdownContent) -> LatexContent:
    """Process inline code spans with proper escaping.

    Args:
        text: Text containing inline code spans

    Returns:
        Text with code spans converted to LaTeX
    """

    def process_code_blocks(match: re.Match[str]) -> str:
        code_content = match.group(1)

        # Check if this is a URL - if so, handle it specially
        is_url = (
            code_content.startswith("http://")
            or code_content.startswith("https://")
            or code_content.startswith("ftp://")
        )

        if is_url:
            # For URLs, use the \url command which handles underscores automatically
            return f"\\url{{{code_content}}}"

        # Check if this code span contains mathematical expressions
        # Mathematical expressions should be protected from seqsplit processing
        has_math_delimiters = "$" in code_content
        has_dollar_paren = "$(" in code_content or "$)" in code_content
        # Also check for protected math placeholders (from earlier math protection step)
        has_protected_math = "XXPROTECTEDMATHXX" in code_content

        # Check if this code span contains LaTeX commands that need special protection
        has_latex_commands = "\\" in code_content and any(
            latex_cmd in code_content
            for latex_cmd in [
                "\\textbf",
                "\\textit",
                "\\emph",
                "\\textsubscript",
                "\\textsuperscript",
                "\\section",
                "\\subsection",
                "\\subsubsection",
                "\\cite",
                "\\ref",
                "\\begin",
                "\\end",
                "\\item",
                "\\href",
                "\\url",
                "\\eqref",
                "\\sidenote",
            ]
        )

        # Check if this is a {{tex:...}} or {{py:...}} reference that should be displayed literally
        # Also handle protected tex block placeholders
        is_block_reference = (
            code_content.startswith("{{")
            and code_content.endswith("}}")
            and ("tex:" in code_content or "py:" in code_content)
        ) or code_content.startswith("XXPROTECTEDTEXBLOCKXX")

        # Check if this code span contains Markdown syntax or special characters that need verbatim display
        # Exclude {{tex:...}} and {{py:...}} references from markdown syntax treatment
        has_markdown_syntax = not is_block_reference and any(
            syntax in code_content
            for syntax in [
                "**",  # Bold markdown
                "*",  # Italic markdown (but not if it's just a single asterisk in regular text)
                "~~",  # Strikethrough markdown
                "`",  # Nested backticks
                "~",  # Subscript syntax (when standalone)
                "^",  # Superscript syntax (when standalone)
                "#",  # Header syntax (single hash)
                "##",  # Header syntax
                "###",  # Header syntax
                "@",  # Citation references
                "<!--",  # HTML comments
                "<",  # HTML tags or special syntax
                ">",  # HTML tags or special syntax
                "[",  # Link syntax
                "]",  # Link syntax
            ]
        )

        if is_block_reference:
            # For {{tex:...}} and {{py:...}} references, use detokenize for robust handling
            # This avoids all the brace escaping issues by treating content literally
            return f"\\texttt{{\\detokenize{{{code_content}}}}}"
        elif has_latex_commands or has_markdown_syntax:
            # For code spans with LaTeX commands or Markdown syntax, use \detokenize for true verbatim display
            # This ensures that \textbf{bold text} or **bold text** appears literally in the PDF
            return f"PROTECTED_DETOKENIZE_START{{{code_content}}}PROTECTED_DETOKENIZE_END"
        elif has_dollar_paren or has_math_delimiters or has_protected_math:
            # For code spans with mathematical content, use \detokenize for robust
            # protection. This prevents LaTeX from interpreting $ as math delimiters
            return f"PROTECTED_DETOKENIZE_START{{{code_content}}}PROTECTED_DETOKENIZE_END"
        else:
            # Handle special LaTeX characters inside code spans using standard escaping
            escaped_content = code_content
            # Hash needs to be escaped in LaTeX as it's used for macro params
            escaped_content = escaped_content.replace("#", "\\#")
            # In texttt, underscores need escaping - use placeholder for safety
            escaped_content = escaped_content.replace("_", "XUNDERSCOREX")

            # For Python inline commands containing mathematical operators, handle quotes first
            if (code_content.startswith("{py:") or "py:" in code_content) and any(
                op in code_content for op in ["*", "//", "^", "+", "-", "%"]
            ):
                # Replace single quotes with double quotes for LaTeX compatibility BEFORE escaping braces
                escaped_content = escaped_content.replace("'", '"')

            # For ALL Python inline commands, use consistent handling
            if code_content.startswith("{py:") or "py:" in code_content:
                # For Python commands, keep braces as-is and only escape other special characters
                py_escaped_content = code_content
                py_escaped_content = py_escaped_content.replace("#", "\\#")
                py_escaped_content = py_escaped_content.replace("_", "XUNDERSCOREX")
                # For Python commands with math operators, also replace quotes
                if any(op in code_content for op in ["*", "//", "^", "+", "-", "%"]):
                    py_escaped_content = py_escaped_content.replace("'", '"')
                # Don't escape braces - let them be literal in texttt
                # Use a protected placeholder to avoid further processing
                return f"PROTECTED_PYTHON_COLOR_START{{{py_escaped_content}}}PROTECTED_PYTHON_COLOR_END"

            # For non-Python commands, handle braces normally
            escaped_content = escaped_content.replace("{", "\\{")
            escaped_content = escaped_content.replace("}", "\\}")

            # For very long code spans (>60 characters), use seqsplit inside texttt
            # to allow line breaks while maintaining monospace formatting
            # BUT only if no LaTeX commands (indicated by backslashes)
            # AND not for Python inline commands which should stay readable
            is_python_command = code_content.startswith("{py:") or code_content.startswith("py:")
            if len(code_content) > 60 and "\\" not in code_content and not is_python_command:
                # Use protected placeholder to prevent escaping of \seqsplit command
                return f"PROTECTED_TEXTTT_SEQSPLIT_START{{{escaped_content}}}PROTECTED_TEXTTT_SEQSPLIT_END"
            else:
                return f"\\texttt{{{escaped_content}}}"

    # Process both double and single backticks
    text = DOUBLE_BACKTICK_PATTERN.sub(process_code_blocks, text)  # Double backticks first
    text = SINGLE_BACKTICK_PATTERN.sub(process_code_blocks, text)  # Then single backticks

    # Convert protected detokenize placeholders to actual LaTeX
    def replace_protected_detokenize(match: re.Match[str]) -> str:
        content = match.group(1)
        return f"\\texttt{{\\detokenize{{{content}}}}}"

    # Use a more robust pattern that handles nested braces
    def find_and_replace_detokenize(text: str) -> str:
        result = []
        i = 0
        while i < len(text):
            # Look for PROTECTED_DETOKENIZE_START{
            start_marker = "PROTECTED_DETOKENIZE_START{"
            if text[i : i + len(start_marker)] == start_marker:
                # Find the matching closing brace
                brace_count = 0
                start = i + len(start_marker)
                j = start
                while j < len(text):
                    if text[j] == "{":
                        brace_count += 1
                    elif text[j] == "}":
                        if brace_count == 0:
                            # Found the matching closing brace
                            content = text[start:j]
                            # Check if this is followed by the end marker
                            end_marker = "}PROTECTED_DETOKENIZE_END"
                            if text[j : j + len(end_marker)] == end_marker:
                                # Handle LaTeX commands with braces specially to avoid \detokenize spacing issues
                                if (
                                    content.startswith("\\")
                                    and "{" in content
                                    and "}" in content
                                    and content.count("\\") == 1
                                ):
                                    # For simple LaTeX commands like \textbf{bold text}, use manual approach
                                    # to avoid the space that \detokenize introduces after \textbf
                                    escaped_content = (
                                        content.replace("\\", "\\textbackslash ")
                                        .replace("{", "\\{")
                                        .replace("}", "\\}")
                                    )
                                    replacement = f"\\texttt{{{escaped_content}}}"
                                else:
                                    # For everything else (markdown syntax, simple text), use \detokenize
                                    # This will handle special characters like #, ~, ^ correctly
                                    # Note: \detokenize may double hash characters in PDF but this is better than backslashes
                                    replacement = f"\\texttt{{\\detokenize{{{content}}}}}"
                                result.append(replacement)
                                i = j + len(end_marker)
                                break
                            else:
                                # No matching end marker, treat as regular content
                                result.append(text[i])
                                i += 1
                                break
                        else:
                            brace_count -= 1
                    j += 1
                else:
                    # No matching brace found, just add the original text
                    result.append(text[i])
                    i += 1
            else:
                result.append(text[i])
                i += 1
        return "".join(result)

    text = find_and_replace_detokenize(text)

    # Convert protected Python color placeholders to actual LaTeX
    text = find_and_replace_python_color(text)

    return text


def apply_bold_italic_formatting(text: MarkdownContent) -> LatexContent:
    """Apply bold and italic formatting while protecting LaTeX commands.

    Args:
        text: Text to format

    Returns:
        Formatted text with LaTeX commands protected
    """

    def safe_bold_replace(match: re.Match[str]) -> str:
        bold_content = match.group(1)
        return f"\\textbf{{{bold_content}}}"

    def safe_italic_replace(match: re.Match[str]) -> str:
        italic_content = match.group(1)
        return f"\\textit{{{italic_content}}}"

    # Replace bold/italic but skip if inside LaTeX commands
    # Split by LaTeX commands and only process text parts
    parts = LATEX_COMMAND_PATTERN.split(text)
    processed_parts: list[str] = []

    for i, part in enumerate(parts):
        if i % 2 == 0:  # This is regular text, not a LaTeX command
            # Apply bold/italic formatting
            part = re.sub(r"\*\*(.+?)\*\*", safe_bold_replace, part)
            part = re.sub(r"\*(.+?)\*", safe_italic_replace, part)
        # If i % 2 == 1, it's a LaTeX command - leave it unchanged
        processed_parts.append(part)

    return "".join(processed_parts)


def protect_bold_outside_texttt(text: MarkdownContent) -> LatexContent:
    """Apply bold formatting only outside texttt blocks.

    Args:
        text: Text to process

    Returns:
        Text with bold formatting applied outside code blocks
    """
    return _apply_formatting_outside_protected_environments(
        text, BOLD_CONSTRAINED_PATTERN, r"\\textbf{\1}", use_full_protection=False
    )


def protect_italic_outside_texttt(text: MarkdownContent) -> LatexContent:
    """Apply italic formatting only outside texttt blocks and LaTeX environments.

    Args:
        text: Text to process

    Returns:
        Text with italic formatting applied outside code blocks and LaTeX environments
    """
    return _apply_formatting_outside_protected_environments(
        text, ITALIC_CONSTRAINED_PATTERN, r"\\textit{\1}", use_full_protection=True
    )


def protect_underline_outside_texttt(text: MarkdownContent) -> LatexContent:
    """Apply underline formatting only outside texttt blocks and specific LaTeX environments.

    Args:
        text: Text to process

    Returns:
        Text with underline formatting applied outside code blocks and protected LaTeX environments
    """
    return _apply_formatting_outside_protected_environments(
        text, UNDERLINE_PATTERN, r"\\underline{\1}", use_full_protection=True
    )


def escape_special_characters(text: MarkdownContent) -> LatexContent:
    """Escape special LaTeX characters in text.

    Args:
        text: Text to escape

    Returns:
        Text with LaTeX special characters escaped
    """
    # First, handle all specific cases that contain listings environments
    # This handles the nested brace issue where regex fails

    # Find all texttt environments that contain listings
    def replace_listings_texttt(text: str) -> str:
        # Simple approach: find texttt blocks with listings and replace with verb
        import re

        # Find all \texttt{...} blocks
        def process_texttt_block(match):
            full_content = match.group(1)

            # If this texttt block contains listings, replace with verb
            if "\\begin{lstlisting}" in full_content:
                # Use verb with a delimiter that's not in the content
                delimiters = [
                    "|",
                    "!",
                    "@",
                    "#",
                    "$",
                    "%",
                    "^",
                    "&",
                    "*",
                    "+",
                    "=",
                    "~",
                ]
                delimiter = "|"
                for d in delimiters:
                    if d not in full_content:
                        delimiter = d
                        break
                return f"\\verb{delimiter}{full_content}{delimiter}"
            else:
                # Return unchanged
                return f"\\texttt{{{full_content}}}"

        # Use re.DOTALL to match across newlines, and handle nested braces properly
        # This pattern handles one level of nested braces without ReDoS vulnerability
        pattern = r"\\texttt\{([^{}]*(?:\{[^}]*\}[^{}]*)*)\}"
        text = re.sub(pattern, process_texttt_block, text, flags=re.DOTALL)

        return text

    text = replace_listings_texttt(text)

    # IMPORTANT: Protect LaTeX commands FIRST before any underscore escaping
    # Protect LaTeX reference commands that should not have underscores escaped
    # These commands use identifiers that often contain underscores (like fig:name_with_underscores)
    protected_latex_commands: dict[str, str] = {}

    def protect_latex_command(match: re.Match[str]) -> str:
        """Replace LaTeX command with placeholder."""
        command = match.group(0)
        placeholder = f"XXPROTECTEDLATEXCOMMANDXX{len(protected_latex_commands)}XXPROTECTEDLATEXCOMMANDXX"
        protected_latex_commands[placeholder] = command
        return placeholder

    # CRITICAL: Protect content that's already been processed by table_processor
    # The table processor uses \detokenize{} for complex cases - don't touch these
    # Protect \texttt{\detokenize{...}} commands (from table processor)
    # Use a more robust pattern that handles nested braces, including triple braces like {{{tex:...}}}
    text = re.sub(
        r"\\texttt\{\\detokenize\{[^{}]*(?:\{[^{}]*(?:\{[^{}]*\}[^{}]*)*\}[^{}]*)*\}\}", protect_latex_command, text
    )

    # Protect standalone \detokenize{...} commands
    text = re.sub(r"\\detokenize\{[^}]*\}", protect_latex_command, text)

    # Protect \includegraphics{} commands
    text = re.sub(r"\\includegraphics\[[^\]]*\]\{[^}]*\}", protect_latex_command, text)

    latex_ref_commands = [
        r"\\ref\{[^}]*\}",  # \ref{fig:name_with_underscores}
        r"\\eqref\{[^}]*\}",  # \eqref{eq:name_with_underscores}
        r"\\label\{[^}]*\}",  # \label{fig:name_with_underscores}
        r"\\pageref\{[^}]*\}",  # \pageref{sec:name_with_underscores}
        r"\\cite\{[^}]*\}",  # \cite{author_2024}
        r"\\citep\{[^}]*\}",  # \citep{author_2024}
        r"\\citet\{[^}]*\}",  # \citet{author_2024}
        r"\\citealt\{[^}]*\}",  # \citealt{author_2024}
        r"\\cref\{[^}]*\}",  # \cref{fig:name_with_underscores} (cleveref)
        r"\\Cref\{[^}]*\}",  # \Cref{fig:name_with_underscores} (cleveref)
    ]

    for pattern in latex_ref_commands:
        text = re.sub(pattern, protect_latex_command, text)

    # Then apply the general function for other cases
    # Escape special characters in texttt commands
    def escape_specials_in_texttt_content(content: str) -> str:
        # Special handling for URL commands - they should not be wrapped in texttt
        if content.startswith("\\url{") and content.endswith("}"):
            # This is already a URL command, return it as-is
            return content

        # Special handling for content that contains URL commands
        if "\\url{" in content:
            # Don't wrap content containing URL commands in texttt to avoid nesting
            # Just return the content as-is since URLs are already properly formatted
            return content

        # Special handling for listings environments - they interfere with texttt
        if "\\begin{lstlisting}" in content or "\\end{lstlisting}" in content:
            # Use verb instead of texttt for listings content
            # Find a delimiter that's not in the content
            delimiters = ["|", "!", "@", "#", "$", "%", "^", "&", "*", "+", "=", "~"]
            delimiter = "|"
            for d in delimiters:
                if d not in content:
                    delimiter = d
                    break
            return f"\\verb{delimiter}{content}{delimiter}"
        # Special handling for detokenize commands - don't escape the backslashes
        elif "\\detokenize{" in content:
            # This content already has detokenize, just return it wrapped in texttt
            return f"\\texttt{{{content}}}"
        # Special handling for LaTeX command protected content - don't escape backslashes
        elif "LATEXCMD_PROTECTED_START{" in content and "}LATEXCMD_PROTECTED_END" in content:
            # Extract the protected content and return it as-is
            start_marker = "LATEXCMD_PROTECTED_START{"
            end_marker = "}LATEXCMD_PROTECTED_END"
            start_idx = content.find(start_marker)
            end_idx = content.find(end_marker)
            if start_idx != -1 and end_idx != -1:
                protected_content = content[start_idx + len(start_marker) : end_idx]
                return f"\\texttt{{{protected_content}}}"
            # Fallback if markers are malformed
            return f"\\texttt{{{content}}}"
        else:
            # For other backslashes, use textbackslash
            # Skip processing if text already contains any form of textbackslash (already escaped)
            if "textbackslash" not in content:
                content = content.replace("\\", "\\textbackslash ")

        # Escape # characters only if not already escaped
        if "\\#" not in content:
            content = content.replace("#", "\\#")
        return f"\\texttt{{{content}}}"

    # Use a more sophisticated approach to handle nested braces
    def find_and_replace_texttt(text: str) -> str:
        result = []
        i = 0
        while i < len(text):
            # Look for \texttt{
            if text[i : i + 8] == "\\texttt{":
                # Find the matching closing brace
                brace_count = 0
                start = i + 8
                j = start
                while j < len(text):
                    if text[j] == "{":
                        brace_count += 1
                    elif text[j] == "}":
                        if brace_count == 0:
                            # Found the matching closing brace
                            content = text[start:j]
                            replacement = escape_specials_in_texttt_content(content)
                            result.append(replacement)
                            i = j + 1
                            break
                        else:
                            brace_count -= 1
                    j += 1
                else:
                    # No matching brace found, just add the original text
                    result.append(text[i])
                    i += 1
            else:
                result.append(text[i])
                i += 1
        return "".join(result)

    text = find_and_replace_texttt(text)

    # Handle underscores carefully - LaTeX is very picky about this
    # We need to escape underscores in text mode but NOT double-escape them

    # Handle remaining underscores in file paths within parentheses
    def escape_file_paths_in_parens(match: re.Match[str]) -> str:
        paren_content = match.group(1)
        # Only escape if it looks like a file path (has extension or
        # is all caps directory)
        if ("." in paren_content and "_" in paren_content) or (
            paren_content.endswith(".md")
            or paren_content.endswith(".bib")
            or paren_content.endswith(".tex")
            or paren_content.endswith(".py")
            or paren_content.endswith(".csv")
        ):
            return f"({paren_content.replace('_', 'XUNDERSCOREX')})"
        return match.group(0)

    text = PARENS_PATTERN.sub(escape_file_paths_in_parens, text)

    # Handle remaining underscores in file names and paths
    # Match common filename patterns: WORD_WORD.ext, word_word.ext, etc.
    def escape_filenames(match: re.Match[str]) -> str:
        filename = match.group(0)
        # Escape underscores in anything that looks like a filename
        return filename.replace("_", "XUNDERSCOREX")

    # Match filenames with extensions
    text = re.sub(
        r"\b[\w]+_[\w._]*\.(md|yml|yaml|bib|tex|py|csv|pdf|png|svg|jpg)\b",
        escape_filenames,
        text,
    )

    # Also match numbered files like 00_CONFIG, 01_MAIN, etc.
    text = UNDERSCORE_ID_PATTERN.sub(escape_filenames, text)

    # Escape percent signs in text (but not in comments that start with %)
    # Use a regex to avoid escaping percent signs at the start of lines (which are comments)
    text = PERCENT_PATTERN.sub(r"\\%", text)

    # Final step: replace all placeholders with properly escaped underscores
    text = text.replace("XUNDERSCOREX", "\\_")

    # Restore protected LaTeX commands after escaping
    for placeholder, original_command in protected_latex_commands.items():
        text = text.replace(placeholder, original_command)

    # Handle special characters that can cause LaTeX issues
    # Escape caret character outside of math mode and texttt blocks
    def escape_carets_outside_protected_contexts(text):
        """Escape carets but not inside LaTeX commands or math mode."""
        # Combine all protection patterns into a single regex
        combined_pattern = (
            r"(\\texttt\{[^}]*\})|"  # \texttt{...}
            r"(\\text\{[^}]*\})|"  # \text{...}
            r"(\$[^$]*\$)|"  # Inline math $...$
            r"(\$\$.*?\$\$)|"  # Display math $$...$$
            r"(\\begin\{equation\}.*?\\end\{equation\})"  # equation environments
        )

        # Split by the combined pattern - protected parts will be in groups
        parts = re.split(combined_pattern, text, flags=re.DOTALL)
        result = []

        for part in parts:
            if part is None or part == "":
                continue

            # Check if this part matches any of our protection patterns
            is_protected = (
                part.startswith("\\texttt{")
                or part.startswith("\\text{")
                or (part.startswith("$") and not part.startswith("$$"))
                or part.startswith("$$")
                or part.startswith("\\begin{equation}")
            )

            if not is_protected:
                # Only escape carets in unprotected parts
                # Only escape isolated carets that aren't already in math mode
                part = CARET_PATTERN.sub(r"\\textasciicircum{}", part)

            result.append(part)

        return "".join(result)

    text = escape_carets_outside_protected_contexts(text)

    # Handle Unicode arrows that can cause LaTeX math mode issues
    # Use safe text replacements that work in all contexts
    text = text.replace("→", " to ")  # Simple text replacement
    text = text.replace("←", " from ")
    text = text.replace("↑", " up ")
    text = text.replace("↓", " down ")

    # Clean up double escaping that may have occurred during table processing
    text = _cleanup_double_escaping_textformatters(text)

    return text


def _cleanup_double_escaping_textformatters(text: str) -> str:
    r"""Clean up double-escaped backslashes in texttt environments.

    Fixes patterns like \\textbackslash{}textbackslash that break LaTeX parsing.
    """
    # Fix the specific pattern of double-escaped backslashes
    # Replace \\textbackslash textbackslash (with space) with just \\textbackslash using pre-compiled patterns
    text = DOUBLE_BACKSLASH_SPACED_PATTERN.sub(r"\\textbackslash ", text)

    # Also try without requiring space after
    text = DOUBLE_BACKSLASH_PATTERN.sub(r"\\textbackslash ", text)

    return text


def restore_protected_seqsplit(text: LatexContent) -> LatexContent:
    """Restore protected seqsplit commands after special character escaping.

    Args:
        text: LaTeX content with protected seqsplit placeholders

    Returns:
        LaTeX content with seqsplit commands restored
    """
    # Handle both escaped and non-escaped versions of the placeholders
    for start_marker, end_marker in [
        ("PROTECTED_TEXTTT_SEQSPLIT_START{", "PROTECTED_TEXTTT_SEQSPLIT_END"),
        (
            "PROTECTED\\_TEXTTT\\_SEQSPLIT\\_START{",
            "PROTECTED\\_TEXTTT\\_SEQSPLIT\\_END",
        ),
    ]:
        while start_marker in text:
            start_pos = text.find(start_marker)
            if start_pos == -1:
                break

            # Find the matching end marker
            content_start = start_pos + len(start_marker)
            brace_count = 1
            pos = content_start

            while pos < len(text) and brace_count > 0:
                if text[pos] == "{":
                    brace_count += 1
                elif text[pos] == "}":
                    brace_count -= 1
                pos += 1

            if brace_count == 0:
                content_end = pos - 1
                content = text[content_start:content_end]

                # Check if this is followed by the end marker
                remaining = text[pos:]
                if remaining.startswith(end_marker):
                    end_marker_end = pos + len(end_marker)

                    # Replace XUNDERSCOREX back to actual underscores
                    content = content.replace("XUNDERSCOREX", "\\_")

                    # Replace with seqsplit
                    replacement = f"\\texttt{{\\seqsplit{{{content}}}}}"
                    text = text[:start_pos] + replacement + text[end_marker_end:]
                else:
                    # If no matching end marker, break to avoid infinite loop
                    break
            else:
                # If braces don't match, break to avoid infinite loop
                break

    return text


def identify_long_technical_identifiers(text: LatexContent, min_length: int = 15) -> LatexContent:
    """Wrap long technical identifiers with seqsplit for better line breaking.

    This function identifies long alphanumeric identifiers (gene names, protein names,
    method names, etc.) and wraps them with LaTeX's seqsplit command to allow
    proper line breaking in scientific documents.

    Args:
        text: Input text that may contain long technical identifiers
        min_length: Minimum length for identifiers to be wrapped (default: 15)

    Returns:
        Text with long identifiers wrapped in seqsplit commands
    """
    # Avoid processing inside LaTeX commands, math mode, or existing seqsplit
    # Pattern to match areas to protect from modification
    protect_pattern = (
        r"(\\[a-zA-Z]+\{[^}]*\})|"  # LaTeX commands like \cite{...}, \texttt{...}
        r"(\$[^$]*\$)|"  # Math mode $...$
        r"(\\seqsplit\{[^}]*\})"  # Existing seqsplit commands
    )

    protected_areas = []
    for match in re.finditer(protect_pattern, text):
        protected_areas.append((match.start(), match.end()))

    def is_protected(start, end):
        """Check if a position range is within a protected area."""
        for pstart, pend in protected_areas:
            if start >= pstart and end <= pend:
                return True
        return False

    # Pattern for long technical identifiers
    # Matches alphanumeric strings with underscores, dots, or mixed case
    identifier_pattern = r"\b[A-Za-z][A-Za-z0-9_.]*[A-Za-z0-9][A-Za-z0-9_.]*\b"

    def replace_identifier(match):
        identifier = match.group()
        start, end = match.span()

        # Skip if in protected area
        if is_protected(start, end):
            return identifier

        # Skip if already wrapped or too short
        if len(identifier) < min_length:
            return identifier

        return f"\\seqsplit{{{identifier}}}"

    result = re.sub(identifier_pattern, replace_identifier, text)
    return result


def wrap_long_strings_in_context(text: LatexContent, min_length: int = 20) -> LatexContent:
    """Wrap long strings that appear after contextual keywords.

    This function looks for long strings that appear after certain contextual
    keywords (like "algorithm", "method", "protocol") and wraps them with seqsplit.

    Args:
        text: Input text to process
        min_length: Minimum length for strings to be wrapped (default: 20)

    Returns:
        Text with contextual long strings wrapped in seqsplit
    """
    # Keywords that often precede technical identifiers
    context_keywords = [
        r"\balgorithm\s+",
        r"\bmethod\s+",
        r"\bprotocol\s+",
        r"\btool\s+",
        r"\bsoftware\s+",
        r"\bpackage\s+",
        r"\blibrary\s+",
    ]

    # Protect same areas as the identifier function
    protect_pattern = (
        r"(\\[a-zA-Z]+\{[^}]*\})|"  # LaTeX commands
        r"(\$[^$]*\$)|"  # Math mode
        r"(\\seqsplit\{[^}]*\})"  # Existing seqsplit
    )

    protected_areas = []
    for match in re.finditer(protect_pattern, text):
        protected_areas.append((match.start(), match.end()))

    def is_protected(start, end):
        """Check if a position range is within a protected area."""
        for pstart, pend in protected_areas:
            if start >= pstart and end <= pend:
                return True
        return False

    result = text
    for keyword in context_keywords:
        # Pattern to match keyword followed by a long identifier
        pattern = f"({keyword})([A-Za-z][A-Za-z0-9_.]*)"

        def replace_contextual(match):
            keyword_part = match.group(1)
            identifier = match.group(2)
            start, end = match.span()

            # Skip if in protected area
            if is_protected(start, end):
                return match.group()

            # Only wrap if identifier is long enough
            if len(identifier) >= min_length:
                return f"{keyword_part}\\seqsplit{{{identifier}}}"
            else:
                return match.group()

        result = re.sub(pattern, replace_contextual, result, flags=re.IGNORECASE)

    return result


def find_and_replace_python_color(text: str) -> str:
    """Convert protected Python color placeholders to actual LaTeX."""
    result = []
    i = 0
    while i < len(text):
        # Look for PROTECTED_PYTHON_COLOR_START{
        start_marker = "PROTECTED_PYTHON_COLOR_START{"
        if text[i : i + len(start_marker)] == start_marker:
            # Find the matching closing brace
            brace_count = 0
            start = i + len(start_marker)
            j = start
            while j < len(text):
                if text[j] == "{":
                    brace_count += 1
                elif text[j] == "}":
                    if brace_count == 0:
                        # Found the matching closing brace
                        content = text[start:j]
                        # Look for the end marker
                        end_marker = "}PROTECTED_PYTHON_COLOR_END"
                        if text[j : j + len(end_marker)] == end_marker:
                            # Replace XUNDERSCOREX back to actual underscores
                            content = content.replace("XUNDERSCOREX", "\\_")
                            # Create Python code with literal braces (no color for now)
                            result.append(f"\\texttt{{{content}}}")
                            i = j + len(end_marker)
                            break
                    else:
                        brace_count -= 1
                j += 1
            else:
                # No matching end found, append character and continue
                result.append(text[i])
                i += 1
        else:
            result.append(text[i])
            i += 1
    return "".join(result)



"""Manuscript utilities for rxiv-maker.

This module provides utility functions that can be called from manuscript Python code
to perform common tasks like figure generation, data processing, and document management.
"""

from .figure_utils import (
    convert_figures_bulk,
    convert_mermaid,
    convert_python_figure,
    convert_r_figure,
)

__all__ = [
    "convert_mermaid",
    "convert_python_figure",
    "convert_r_figure",
    "convert_figures_bulk",
]



"""Rxiv-Maker Python Package.

A comprehensive toolkit for automated scientific article generation and building.

This package provides a comprehensive toolkit for preparing academic manuscripts
with LaTeX-quality output, integrated bibliography management, figure generation,
and validation systems.

Main Components:
- Services: High-level business logic and orchestration
- Core: Central managers, cache, and configuration
- Engines: Document processing and build operations
- CLI: Command-line interface and user interactions
- Utils: Common utilities and platform helpers
- Validators: Comprehensive manuscript validation

Quick Start:
    from rxiv_maker import ManuscriptService, ValidationService

    # Discover and process manuscript
    ms = ManuscriptService()
    result = ms.discover_manuscript()

    # Validate manuscript
    vs = ValidationService()
    validation = vs.validate_manuscript_comprehensive()
"""

from .__version__ import __version__

# Service Layer Facade - Most commonly needed for external use
try:
    from .services import (  # noqa: F401
        BuildService,
        ConfigurationService,
        ManuscriptService,
        PublicationService,
        ValidationService,
    )

    # Essential exceptions for error handling
    from .services.base import ProcessingError, ServiceError, ServiceResult, ValidationError  # noqa: F401

    SERVICES_AVAILABLE = True
except ImportError:
    # Services may not be available in all environments
    SERVICES_AVAILABLE = False

# Core Utilities Facade - Commonly used across modules
try:
    from .utils.platform import platform_detector  # noqa: F401
    from .utils.unicode_safe import get_safe_icon, safe_console_print, safe_print  # noqa: F401

    UTILS_AVAILABLE = True
except ImportError:
    UTILS_AVAILABLE = False

# Cache Facade - Frequently accessed
try:
    from .core.cache import AdvancedCache, BibliographyCache, DOICache, get_cache_dir  # noqa: F401

    CACHE_AVAILABLE = True
except ImportError:
    CACHE_AVAILABLE = False


# Maintain backward compatibility
def get_version():
    """Get version information."""
    return __version__


def get_versions():
    """Get version information in versioneer-compatible format."""
    return {"version": __version__}


__author__ = "Rxiv-Maker Contributors"
__description__ = "Academic manuscript preparation toolkit"

# Dynamic __all__ based on available components
__all__ = [
    # Version info
    "__version__",
    "__author__",
    "__description__",
    "get_version",
    "get_versions",
]

if SERVICES_AVAILABLE:
    __all__.extend(
        [
            "ManuscriptService",
            "ValidationService",
            "BuildService",
            "PublicationService",
            "ConfigurationService",
            "ServiceError",
            "ValidationError",
            "ProcessingError",
            "ServiceResult",
        ]
    )

if UTILS_AVAILABLE:
    __all__.extend(
        [
            "safe_print",
            "safe_console_print",
            "get_safe_icon",
            "platform_detector",
        ]
    )

if CACHE_AVAILABLE:
    __all__.extend(
        [
            "get_cache_dir",
            "AdvancedCache",
            "BibliographyCache",
            "DOICache",
        ]
    )



"""DOCX export command for rxiv-maker CLI."""

import rich_click as click
from rich.console import Console

from ...core.logging_config import get_logger
from ...exporters.docx_exporter import DocxExporter

logger = get_logger()
console = Console()


@click.command(context_settings={"help_option_names": ["-h", "--help"]})
@click.argument(
    "manuscript_path",
    type=click.Path(exists=True, file_okay=False),
    required=False,
    metavar="[MANUSCRIPT_PATH]",
)
@click.option(
    "--resolve-dois",
    "-r",
    is_flag=True,
    help="Attempt to resolve missing DOIs from metadata",
)
@click.option(
    "--no-footnotes",
    is_flag=True,
    help="Disable references section (citations only)",
)
@click.option("--verbose", "-v", is_flag=True, help="Enable verbose output")
@click.option("--quiet", "-q", is_flag=True, help="Suppress non-essential output")
def docx(
    manuscript_path: str | None,
    resolve_dois: bool,
    no_footnotes: bool,
    verbose: bool,
    quiet: bool,
) -> None:
    """Export manuscript to DOCX format for collaborative review.

    Generates a Word document with numbered citations, embedded figures,
    and a complete references section with DOI links. Output is automatically
    saved to the manuscript directory with the same naming pattern as the PDF.

    **MANUSCRIPT_PATH**: Directory containing manuscript files.
    Defaults to MANUSCRIPT/

    **Output**: Automatically saved to MANUSCRIPT/YEAR__lastname_et_al__rxiv.docx

    ## Examples

    **Basic export:**

        $ rxiv docx

    **Export from custom directory:**

        $ rxiv docx MY_PAPER/

    **With DOI resolution for missing entries:**

        $ rxiv docx --resolve-dois

    **Without references section (citations only):**

        $ rxiv docx --no-footnotes
    """
    try:
        # Configure logging
        if verbose:
            logger.set_level("DEBUG")
        elif quiet:
            logger.set_level("WARNING")

        # Set manuscript path
        manuscript_path = manuscript_path or "MANUSCRIPT"

        # Create exporter
        if not quiet:
            console.print("[cyan]📄 Exporting manuscript to DOCX...[/cyan]")

        exporter = DocxExporter(
            manuscript_path=manuscript_path,
            resolve_dois=resolve_dois,
            include_footnotes=not no_footnotes,
        )

        # Perform export
        docx_path = exporter.export()

        # Success message
        if not quiet:
            console.print(f"[green]✅ DOCX exported:[/green] {docx_path}")

    except FileNotFoundError as e:
        console.print(f"[red]❌ Error:[/red] {e}")
        raise click.Abort() from e
    except Exception as e:
        if verbose:
            logger.error(f"DOCX export failed: {e}")
        console.print(f"[red]❌ Export failed:[/red] {e}")
        raise click.Abort() from e



"""Configuration management command implementations for rxiv-maker CLI."""

from pathlib import Path
from typing import Optional

import click

from .base import BaseCommand, CommandExecutionError


class ConfigInitCommand(BaseCommand):
    """Config init command implementation using the framework."""

    def setup_common_options(self, ctx: click.Context, manuscript_path: Optional[str] = None) -> None:
        """Override setup since config commands don't need path manager."""
        pass

    def check_engine_support(self) -> None:
        """Override engine check since config commands don't need engine."""
        pass

    def execute_operation(self, template: str = "default", force: bool = False, output: Optional[str] = None) -> None:
        """Execute config initialization."""
        import yaml

        from rxiv_maker.core.managers.config_manager import get_config_manager

        try:
            config_manager = get_config_manager()

            if output:
                output_path = Path(output)
                # Custom output path
                if output_path.exists() and not force:
                    self.error_message(f"Configuration file already exists: {output_path}")
                    self.console.print("Use --force to overwrite", style="yellow")
                    raise CommandExecutionError("Configuration file already exists")

                # Get template config and write to custom path
                template_config = config_manager._get_config_template(template)

                with open(output_path, "w", encoding="utf-8") as f:
                    yaml.dump(template_config, f, default_flow_style=False, sort_keys=False)

                config_path = output_path
            else:
                # Use default initialization
                config_path = config_manager.init_config(template, force)

            self.success_message(f"Configuration initialized: {config_path}")
            self.console.print(f"📋 Template: {template}", style="blue")
            self.console.print("💡 Run 'rxiv config validate' to check your configuration", style="dim")

        except ValueError as e:
            self.error_message(str(e))
            raise CommandExecutionError(str(e)) from e
        except Exception as e:
            self.error_message(f"Failed to initialize configuration: {e}")
            raise CommandExecutionError(f"Configuration initialization failed: {e}") from e


class ConfigValidateCommand(BaseCommand):
    """Config validate command implementation using the framework."""

    def setup_common_options(self, ctx: click.Context, manuscript_path: Optional[str] = None) -> None:
        """Override setup since config commands don't need path manager."""
        pass

    def check_engine_support(self) -> None:
        """Override engine check since config commands don't need engine."""
        pass

    def execute_operation(
        self, config_path: Optional[str] = None, output_format: str = "table", strict: bool = False
    ) -> None:
        """Execute config validation."""
        import json

        from rxiv_maker.config.validator import ConfigValidator
        from rxiv_maker.core.managers.config_manager import get_config_manager

        try:
            config_manager = get_config_manager()
            validator = ConfigValidator()

            self.console.print("🔍 Validating configuration...", style="blue")
            self.console.print("=" * 50)

            config_path_obj = Path(config_path) if config_path else None

            # Validate manuscript configuration
            if config_path_obj:
                self.console.print(f"📄 Validating: {config_path_obj}")
                config_validation = config_manager.validate_config(config_path_obj)
            else:
                self.console.print("📄 Searching for configuration file...")
                config_validation = config_manager.validate_config()

            # Validate CLI arguments
            cli_validation = validator.validate_cli_arguments(
                {"config_path": str(config_path_obj) if config_path_obj else None, "strict": strict}, "validate"
            )

            # Validate environment and project structure
            env_validation = validator.validate_environment_config()
            project_validation = validator.validate_project_structure(Path.cwd())

            # Combine all validation results
            all_results = {
                "configuration": config_validation,
                "cli_arguments": cli_validation,
                "environment": env_validation,
                "project_structure": project_validation,
            }

            if output_format == "json":
                self.console.print(json.dumps(all_results, indent=2, default=str))
            else:
                self._print_validation_results(all_results, strict)

            # Check for errors
            has_errors = any(not result.get("valid", True) for result in all_results.values())

            if has_errors:
                if strict:
                    raise CommandExecutionError("Configuration validation failed (strict mode)")
                else:
                    self.console.print("⚠️  Configuration has warnings/errors", style="yellow")
            else:
                self.success_message("Configuration validation passed!")

        except Exception as e:
            self.error_message(f"Configuration validation failed: {e}")
            raise CommandExecutionError(f"Validation failed: {e}") from e

    def _print_validation_results(self, results: dict, strict: bool) -> None:
        """Print validation results in table format."""
        for category, result in results.items():
            valid = result.get("valid", True)
            status = "✅ Valid" if valid else "❌ Invalid"
            self.console.print(f"{category.title()}: {status}")


class ConfigGetCommand(BaseCommand):
    """Config get/set command implementation using the framework."""

    def setup_common_options(self, ctx: click.Context, manuscript_path: Optional[str] = None) -> None:
        """Override setup since config commands don't need path manager."""
        pass

    def check_engine_support(self) -> None:
        """Override engine check since config commands don't need engine."""
        pass

    def execute_operation(
        self, key: str, value: Optional[str] = None, config_path: Optional[str] = None, value_type: str = "string"
    ) -> None:
        """Execute config get/set operation."""
        from rxiv_maker.core.managers.config_manager import get_config_manager

        try:
            config_manager = get_config_manager()
            config_path_obj = Path(config_path) if config_path else None

            if value is None:
                # Get value
                config_value = config_manager.get_config_value(key, config_path=config_path_obj)

                if config_value is None:
                    self.error_message(f"Configuration key '{key}' not found")
                    return

                self.console.print(f"📝 {key}: {config_value}", style="green")

                if isinstance(config_value, dict):
                    self.console.print("\n🔍 Nested configuration:", style="blue")
                    for nested_key, nested_value in config_value.items():
                        self.console.print(f"   {key}.{nested_key}: {nested_value}")
            else:
                # Set value with type conversion
                converted_value = self._convert_value(value, value_type)
                updated_path = config_manager.set_config_value(key, converted_value, config_path_obj)

                self.success_message(f"Updated {key} = {converted_value}")
                self.console.print(f"📄 Configuration file: {updated_path}", style="blue")

        except Exception as e:
            self.error_message(f"Configuration operation failed: {e}")
            raise CommandExecutionError(f"Configuration operation failed: {e}") from e

    def _convert_value(self, value: str, value_type: str):
        """Convert string value to appropriate type."""
        import json

        if value_type == "int":
            return int(value)
        elif value_type == "float":
            return float(value)
        elif value_type == "bool":
            return value.lower() in ("true", "1", "yes", "on")
        elif value_type == "json":
            return json.loads(value)
        else:
            return value


class ConfigShowCommand(BaseCommand):
    """Config show command implementation using the framework."""

    def setup_common_options(self, ctx: click.Context, manuscript_path: Optional[str] = None) -> None:
        """Override setup since config commands don't need path manager."""
        pass

    def check_engine_support(self) -> None:
        """Override engine check since config commands don't need engine."""
        pass

    def execute_operation(
        self, output_format: str = "table", config_path: Optional[str] = None, include_defaults: bool = False
    ) -> None:
        """Execute config show operation."""
        import json

        import click
        import yaml

        from rxiv_maker.core.managers.config_manager import get_config_manager

        try:
            config_manager = get_config_manager()
            config_path_obj = Path(config_path) if config_path else None

            config = config_manager.get_config(config_path_obj, include_defaults=include_defaults)

            if output_format == "json":
                self.console.print(json.dumps(config, indent=2, default=str))
            elif output_format == "yaml":
                yaml.dump(config, click.get_text_stream("stdout"), default_flow_style=False)
            else:
                self._print_config_table(config, include_defaults)

        except Exception as e:
            self.error_message(f"Failed to show configuration: {e}")
            raise CommandExecutionError(f"Failed to show configuration: {e}") from e

    def _print_config_table(self, config: dict, include_defaults: bool) -> None:
        """Print config in table format."""
        for key, value in config.items():
            self.console.print(f"{key}: {value}")


class ConfigExportCommand(BaseCommand):
    """Config export command implementation using the framework."""

    def setup_common_options(self, ctx: click.Context, manuscript_path: Optional[str] = None) -> None:
        """Override setup since config commands don't need path manager."""
        pass

    def check_engine_support(self) -> None:
        """Override engine check since config commands don't need engine."""
        pass

    def execute_operation(
        self,
        output: str,
        export_format: str = "yaml",
        include_defaults: bool = False,
        config_path: Optional[str] = None,
    ) -> None:
        """Execute config export operation."""
        from rxiv_maker.core.managers.config_manager import get_config_manager

        try:
            config_manager = get_config_manager()
            output_path = Path(output)
            config_path_obj = Path(config_path) if config_path else None

            exported_path = config_manager.export_config(output_path, export_format, include_defaults, config_path_obj)

            self.success_message(f"Configuration exported to: {exported_path}")
            self.console.print(f"📊 Format: {export_format.upper()}", style="blue")

            if include_defaults:
                self.console.print("📝 Includes default values", style="dim")
            else:
                self.console.print("📝 Custom values only", style="dim")

        except Exception as e:
            self.error_message(f"Export failed: {e}")
            raise CommandExecutionError(f"Configuration export failed: {e}") from e


class ConfigMigrateCommand(BaseCommand):
    """Config migrate command implementation using the framework."""

    def setup_common_options(self, ctx: click.Context, manuscript_path: Optional[str] = None) -> None:
        """Override setup since config commands don't need path manager."""
        pass

    def check_engine_support(self) -> None:
        """Override engine check since config commands don't need engine."""
        pass

    def execute_operation(
        self, from_version: str, to_version: str, config_path: Optional[str] = None, backup: bool = True
    ) -> None:
        """Execute config migration operation."""
        from rxiv_maker.core.managers.config_manager import get_config_manager

        try:
            config_manager = get_config_manager()
            config_path_obj = Path(config_path) if config_path else None

            self.console.print(f"🔄 Migrating configuration: {from_version} → {to_version}", style="blue")

            if backup:
                self.console.print("💾 Backup will be created automatically", style="dim")

            migrated_path = config_manager.migrate_config(from_version, to_version, config_path_obj)

            self.success_message(f"Configuration migrated: {migrated_path}")
            self.console.print("🔍 Please review the migrated configuration", style="yellow")
            self.console.print("💡 Run 'rxiv config validate' to verify the migration", style="dim")

        except Exception as e:
            self.error_message(f"Migration failed: {e}")
            raise CommandExecutionError(f"Configuration migration failed: {e}") from e


class ConfigListCommand(BaseCommand):
    """Config list command implementation using the framework."""

    def setup_common_options(self, ctx: click.Context, manuscript_path: Optional[str] = None) -> None:
        """Override setup since config commands don't need path manager."""
        pass

    def check_engine_support(self) -> None:
        """Override engine check since config commands don't need engine."""
        pass

    def execute_operation(self) -> None:
        """Execute config list operation."""
        from rich.table import Table

        from rxiv_maker.core.repo_config import get_repo_config
        from rxiv_maker.core.repository import RepositoryManager

        try:
            files = []

            # Check for global repository config first
            repo_config = get_repo_config()
            repo_config_path = repo_config.config_path
            # Add .yaml extension for display (actual file has no extension)
            repo_config_display = f"{repo_config_path}.yaml"
            if repo_config_path.exists():
                stat = repo_config_path.stat()
                files.append(
                    {
                        "path": repo_config_display,
                        "exists": True,
                        "size": stat.st_size,
                        "type": "Repository",
                        "repo_name": "-",
                    }
                )
            else:
                files.append(
                    {
                        "path": repo_config_display,
                        "exists": False,
                        "size": 0,
                        "type": "Repository",
                        "repo_name": "-",
                    }
                )

            # Discover all manuscript repositories
            repo_manager = RepositoryManager()
            try:
                parent_dir = repo_config.parent_dir
                if parent_dir.exists():
                    repos = repo_manager.discover_repositories(parent_dir)

                    # Add config file for each repository
                    for repo in repos:
                        manuscript_config = repo.manuscript_dir / "00_CONFIG.yml"
                        if manuscript_config.exists():
                            stat = manuscript_config.stat()
                            files.append(
                                {
                                    "path": str(manuscript_config),
                                    "exists": True,
                                    "size": stat.st_size,
                                    "type": "Manuscript",
                                    "repo_name": repo.name,
                                }
                            )
                        else:
                            files.append(
                                {
                                    "path": str(manuscript_config),
                                    "exists": False,
                                    "size": 0,
                                    "type": "Manuscript",
                                    "repo_name": repo.name,
                                }
                            )
            except Exception:
                # If no parent dir configured or error, just show current directory
                manuscript_config = Path.cwd() / "00_CONFIG.yml"
                if manuscript_config.exists():
                    stat = manuscript_config.stat()
                    files.append(
                        {
                            "path": str(manuscript_config),
                            "exists": True,
                            "size": stat.st_size,
                            "type": "Manuscript",
                            "repo_name": "current",
                        }
                    )
                else:
                    files.append(
                        {
                            "path": str(manuscript_config),
                            "exists": False,
                            "size": 0,
                            "type": "Manuscript",
                            "repo_name": "current",
                        }
                    )

            table = Table(title="Configuration Files", expand=True)
            table.add_column("Type", style="yellow", no_wrap=True)
            table.add_column("Repo", style="magenta", no_wrap=True)
            table.add_column("Path", style="cyan", overflow="fold")
            table.add_column("Exists", justify="center", no_wrap=True)
            table.add_column("Size", no_wrap=True)

            for file_info in files:
                exists = "✓" if file_info["exists"] else "✗"
                size = f"{file_info.get('size', 0)} bytes" if file_info["exists"] else "-"
                table.add_row(file_info["type"], file_info["repo_name"], file_info["path"], exists, size)

            self.console.print(table)
            self.console.print()
            self.console.print(
                f"[dim]Found {len([f for f in files if f['type'] == 'Manuscript'])} manuscript config(s)[/dim]"
            )
            self.console.print("[dim]Manuscript configs: */MANUSCRIPT/00_CONFIG.yml in each repository[/dim]")
            self.console.print("[dim]Repository config: ~/.rxiv-maker/config for global settings[/dim]")
            self.console.print()

        except Exception as e:
            self.error_message(f"Failed to list configuration files: {e}")
            raise CommandExecutionError(f"Failed to list configuration files: {e}") from e


__all__ = [
    "ConfigInitCommand",
    "ConfigValidateCommand",
    "ConfigGetCommand",
    "ConfigShowCommand",
    "ConfigExportCommand",
    "ConfigMigrateCommand",
    "ConfigListCommand",
]



"""Test for trailing slash issue in manuscript paths (Issue #100)."""

import os

import pytest

from rxiv_maker.engines.operations.build_manager import BuildManager


class TestTrailingSlashRegression:
    """Test cases for Issue #100 - BibTeX returned error code 1 due to trailing slash paths."""

    def setup_manuscript_dir(self, temp_dir, name):
        """Set up a minimal manuscript directory for testing."""
        manuscript_dir = temp_dir / name
        manuscript_dir.mkdir(parents=True, exist_ok=True)

        # Create minimal required files
        (manuscript_dir / "01_MAIN.md").write_text("# Test Manuscript")
        (manuscript_dir / "00_CONFIG.yml").write_text("title: Test")

        return manuscript_dir

    def test_manuscript_name_trailing_slash_paths(self, temp_dir):
        """Test that BuildManager handles trailing slashes correctly in manuscript paths."""

        test_cases = [
            ("CCT8_paper", "CCT8_paper"),  # No slash
            ("CCT8_paper/", "CCT8_paper"),  # Single slash (Guillaume's case)
            ("CCT8_paper//", "CCT8_paper"),  # Double slash
            ("my_project/", "my_project"),  # Different name with slash
        ]

        for input_path_suffix, expected_name in test_cases:
            # Create manuscript directory
            manuscript_dir = self.setup_manuscript_dir(temp_dir, expected_name)

            # Build full path with the suffix (slash variations)
            if input_path_suffix.endswith("/"):
                # For paths with trailing slash, use the directory + slash
                manuscript_path = str(manuscript_dir) + input_path_suffix[len(expected_name) :]
            else:
                manuscript_path = str(manuscript_dir)

            output_dir = temp_dir / "output" / expected_name

            # Create BuildManager instance
            build_manager = BuildManager(
                manuscript_path=manuscript_path, output_dir=str(output_dir), skip_validation=True
            )

            # Verify the manuscript name is extracted correctly
            assert build_manager.manuscript_name == expected_name, (
                f"Failed for input '{input_path_suffix}': "
                f"expected '{expected_name}', got '{build_manager.manuscript_name}'"
            )

            # Verify output file paths use the correct name
            assert build_manager.output_tex.name == f"{expected_name}.tex"
            assert build_manager.output_pdf.name == f"{expected_name}.pdf"

    def test_environment_variable_setting_with_trailing_slash(self, temp_dir):
        """Test that MANUSCRIPT_PATH environment variable is set correctly with trailing slashes."""

        manuscript_dir = self.setup_manuscript_dir(temp_dir, "test_project")
        output_dir = temp_dir / "output"

        # Test with trailing slash
        manuscript_path_with_slash = str(manuscript_dir) + "/"

        build_manager = BuildManager(
            manuscript_path=manuscript_path_with_slash, output_dir=str(output_dir), skip_validation=True
        )

        # Simulate the environment variable setting logic from generate_tex_files
        normalized_path = build_manager.manuscript_path.rstrip("/")
        manuscript_name = os.path.basename(normalized_path)
        if not manuscript_name or manuscript_name in (".", ".."):
            manuscript_name = "MANUSCRIPT"

        # Should extract correct name despite trailing slash
        assert manuscript_name == "test_project"

    def test_edge_cases_for_invalid_paths(self, temp_dir):
        """Test edge cases that should default to MANUSCRIPT."""

        output_dir = temp_dir / "output"

        # Test cases that should succeed
        valid_edge_cases = [
            ("", "MANUSCRIPT"),  # Empty string
            (".", "MANUSCRIPT"),  # Current directory
        ]

        for input_path, expected_name in valid_edge_cases:
            build_manager = BuildManager(manuscript_path=input_path, output_dir=str(output_dir), skip_validation=True)

            assert build_manager.manuscript_name == expected_name, (
                f"Failed for input '{input_path}': expected '{expected_name}', got '{build_manager.manuscript_name}'"
            )

        # Test that ".." path is properly blocked for security
        from rxiv_maker.core.path_manager import PathResolutionError

        # Don't skip validation for security checks
        with pytest.raises(PathResolutionError, match="Path traversal not allowed"):
            BuildManager(manuscript_path="..", output_dir=str(output_dir), skip_validation=False)

    def test_guillaume_exact_case(self, temp_dir):
        """Test Guillaume's exact use case: 'rxiv pdf CCT8_paper/'."""

        # Set up exactly like Guillaume's case
        manuscript_dir = self.setup_manuscript_dir(temp_dir, "CCT8_paper")
        manuscript_path = str(manuscript_dir) + "/"  # With trailing slash
        output_dir = manuscript_dir / "output"

        build_manager = BuildManager(manuscript_path=manuscript_path, output_dir=str(output_dir), skip_validation=True)

        # This should work correctly now
        assert build_manager.manuscript_name == "CCT8_paper"
        assert build_manager.output_tex.name == "CCT8_paper.tex"
        assert build_manager.output_pdf.name == "CCT8_paper.pdf"

        # Verify that the expected LaTeX file name matches what will be generated
        # This ensures LaTeX compilation will find the right file
        normalized_path = manuscript_path.rstrip("/")
        manuscript_name = os.path.basename(normalized_path)
        if not manuscript_name or manuscript_name in (".", ".."):
            manuscript_name = "MANUSCRIPT"

        assert manuscript_name == "CCT8_paper"
        assert manuscript_name == build_manager.manuscript_name
